@@ -21,7 +21,7 @@ import logging
 import os
 import tempfile
 import time
-from typing import Any
+from typing import Any, Optional
 import warnings
 
 from jax._src import compilation_cache
@@ -34,7 +34,6 @@ from jax._src import traceback_util
 from jax._src.interpreters import mlir
 from jax._src.lib import xla_client as xc
 from jax._src.lib.mlir import ir
-from jax._src.xla_bridge import process_count
 import numpy as np
 
 
@@ -218,9 +217,11 @@ def backend_compile(
     options: xc.CompileOptions,
     host_callbacks: Sequence[Any],
 ) -> xc.LoadedExecutable:
-  # Convert ir.Module to a string representation, unless the
-  # back-end expliclity flags the ability to handle a module directly
-  # (avoiding the overhead of back and forth conversions)
+  # Convert ir.Module to a string representation, unless the backend
+  # explicitly flags the ability to handle a module directly (avoiding the
+  # overhead of back and forth conversions).
+  # TODO(slebedev): Change the backend.compile() to accept ir.Module.
+  built_c: Any
   if getattr(backend, "needs_str_ir", True):
     built_c = mlir.module_to_bytecode(module)
   else:
@@ -242,6 +243,7 @@ def compile_or_get_cached(
     devices: np.ndarray,
     compile_options: xc.CompileOptions,
     host_callbacks: Sequence[Any],
+    pgle_profiler: profiler.PGLEProfiler | None = None,
 ) -> xc.LoadedExecutable:
   sym_name = computation.operation.attributes['sym_name']
   module_name = ir.StringAttr(sym_name).value
@@ -277,6 +279,50 @@ def compile_or_get_cached(
     return backend_compile(backend, computation, compile_options,
                            host_callbacks)
 
+  is_multi_process = (
+      len({device.process_index for device in devices.flatten()}) > 1)
+  min_device_process_id = (
+      min(devices.flatten(), key=lambda device: device.id).process_index)
+
+  # When PGLE is enabled there might be 3 types of situations:
+  # 1. PGLE profiled module (the one which was recompiled with FDO profile) is
+  # in the persistent cache. In this case the module should be returned from
+  # cache and PGLE should be disabled for this module. Is module is stored in
+  # the persistent cache under the "pgle_profiled_module_key" which calculated
+  # with replacing FDO profile with flag which identify that module were PGLE
+  # profiled.
+  # 2. PGLE profiled module is not in the persistent cache and the module is
+  # getting built with an FDO profile. In this case we need to share FDO profile
+  # with other processes and store the result under the
+  # "pgle_profiled_module_key" so later in case 1 we will be able to find the
+  # module.
+  # 3. PGLE profiled module is not in the persistent cache and the module is
+  # getting compiled to be PGLEd (FDO profile is empty). In this case we need to
+  # simply return the non-PGLE profiled module from the persistent cache.
+  if (config.enable_pgle.value
+      and config.pgle_profiling_runs.value > 0):
+    fdo_profile = compile_options.executable_build_options.fdo_profile
+    compile_options.executable_build_options.fdo_profile = b"pgle profiled"
+
+    pgle_profiled_module_key = compilation_cache.get_cache_key(
+        computation, devices, compile_options, backend)
+    compile_options.executable_build_options.fdo_profile = fdo_profile
+
+    if _is_executable_in_cache(backend, pgle_profiled_module_key):
+      # Load PGLE profiled module from the persistent cache.
+      cache_key = pgle_profiled_module_key
+      if pgle_profiler is not None:
+        pgle_profiler.disable()
+    elif fdo_profile is not None and len(fdo_profile) > 0:
+      # Store module under PGLE profiled module cache key.
+      cache_key = pgle_profiled_module_key
+      if is_multi_process and distributed.global_state.client is not None:
+        compile_options.executable_build_options.fdo_profile = _share_fdo_profiles(
+          computation, devices, compile_options, backend,
+          distributed.global_state.client,
+          min_device_process_id
+        )
+
   cache_retrieval_start = time.monotonic()
   retrieved_executable, retrieved_compile_time = _cache_read(
       module_name, cache_key, compile_options, backend)
@@ -296,8 +342,8 @@ def compile_or_get_cached(
 
     return retrieved_executable
   elif (
-      process_count() > 1
-      and config.share_binary_between_hosts.value
+      config.share_binary_between_hosts.value
+      and is_multi_process
       and distributed.global_state.client is not None
       # Host callbacks are currently baked into the HLO module so we cant share
       # them.
@@ -311,10 +357,11 @@ def compile_or_get_cached(
         distributed.global_state.client,
         module_name,
         cache_key,
+        min_device_process_id
     )
   elif (
-      process_count() > 1
-      and config.share_autotune_config_between_hosts.value
+      config.share_autotune_config_between_hosts.value
+      and is_multi_process
       and distributed.global_state.client is not None
   ):
     return _compile_and_write_autotune_config(
@@ -325,6 +372,7 @@ def compile_or_get_cached(
         distributed.global_state.client,
         module_name,
         cache_key,
+        min_device_process_id
     )
   else:
     return _compile_and_write_cache(
@@ -336,9 +384,61 @@ def compile_or_get_cached(
         cache_key,
     )
 
+# The process that has the lowest device ID should share FDO profile before
+# compilation with other processes.
+def _share_fdo_profiles(
+    computation: ir.Module,
+    devices: np.ndarray,
+    compile_options: xc.CompileOptions,
+    backend: xc.Client,
+    global_client: lib.xla_extension.DistributedRuntimeClient,
+    min_process_id
+) -> Optional[bytes]:
+  sym_name = computation.operation.attributes['sym_name']
+  module_name = ir.StringAttr(sym_name).value
+  fdo_profile = compile_options.executable_build_options.fdo_profile
+  if fdo_profile is None or len(fdo_profile) == 0:
+    return fdo_profile
 
-# The process with id 0 should compile the module and write an autotune config
-# to the K-V storage.
+  compile_options.executable_build_options.fdo_profile = b""
+  profile_key = (
+      compilation_cache.get_cache_key(
+          computation, devices, compile_options, backend
+      )
+      + "_fdo_sync"
+  )
+  if profile_key in _share_fdo_profiles.modules_profiles:
+    return _share_fdo_profiles.modules_profiles[profile_key]
+
+  share_timeout = config.share_binary_between_hosts_timeout_ms.value
+  if distributed.global_state.process_id == min_process_id:
+    logger.debug(
+        "Sharing FDO profile: %s. For module %s. Process %d.",
+        fdo_profile,
+        module_name,
+        min_process_id,
+    )
+    global_client.key_value_set_bytes(profile_key, fdo_profile)
+  else:
+    logger.debug(
+        "Waiting for FDO profile: %s. For module %s. Should be set by process %d.",
+        fdo_profile,
+        module_name,
+        min_process_id,
+    )
+    fdo_profile = global_client.blocking_key_value_get_bytes(
+        profile_key, share_timeout
+    )
+
+  _share_fdo_profiles.modules_profiles[profile_key] = fdo_profile
+  return fdo_profile
+
+
+_share_fdo_profiles.modules_profiles = {}
+
+
+# The process with the first_process_id should compile the module and write an
+# autotune config to the K-V storage.
 def _compile_and_write_autotune_config(
     backend: xc.Client,
     computation: ir.Module,
@@ -347,6 +447,7 @@ def _compile_and_write_autotune_config(
     global_client: lib.xla_extension.DistributedRuntimeClient,
     module_name: str,
     cache_key: str,
+    first_process_id: int
 ) -> xc.LoadedExecutable:
   share_timeout = config.share_binary_between_hosts_timeout_ms.value
   debug_options = compile_options.executable_build_options.debug_options
@@ -374,9 +475,10 @@ def _compile_and_write_autotune_config(
         cache_key,
     )
 
-  if distributed.global_state.process_id == 0:
+  if distributed.global_state.process_id == first_process_id:
     debug_options.xla_gpu_dump_autotune_results_to = autotune_tmp_file
-    logger.debug("Compiling and dumping autotune for module: %s", module_name)
+    logger.debug("Process %d compiling and dumping autotune for module: %s",
+                 first_process_id, module_name)
     executable = _compile_and_write_cache(
         backend,
         computation,
@@ -404,9 +506,11 @@ def _compile_and_write_autotune_config(
     )
   else:
     logger.debug(
-        "Compiling module %s, waiting for config to be shared by cache_key %s",
+        "Compiling module %s, waiting for config to be shared by cache_key %s"
+        "from process %d",
         module_name,
         cache_key,
+        first_process_id
     )
     autotune_config = global_client.blocking_key_value_get_bytes(
         cache_key, share_timeout
@@ -439,10 +543,8 @@ def _compile_and_write_autotune_config(
 
 _compile_and_write_autotune_config.autotune_configs_dir = None
 
-# The process with id 0 should compile the module and write it to the K-V
-# storage.
-# TODO: In case when the process with id 0 is not participating in computation
-# we need to choose another process to compile the module.
+# The process with the first_process_id should compile the module and write it
+# to the K-V storage.
 def _compile_and_share_module(
     backend: xc.Client,
     computation: ir.Module,
@@ -451,6 +553,7 @@ def _compile_and_share_module(
     global_client: lib.xla_extension.DistributedRuntimeClient,
     module_name: str,
     cache_key: str,
+    first_process_id: int
 ) -> xc.LoadedExecutable:
   share_timeout = config.share_binary_between_hosts_timeout_ms.value
 
@@ -459,7 +562,9 @@ def _compile_and_share_module(
   if cache_key in _compile_and_share_module.modules_cache:
     return _compile_and_share_module.modules_cache[cache_key]
 
-  if distributed.global_state.process_id == 0:
+  if distributed.global_state.process_id == first_process_id:
+    logger.debug("Process %d compiling and sharing module: %s",
+                 first_process_id, module_name)
     executable = _compile_and_write_cache(
         backend,
         computation,
@@ -474,6 +579,8 @@ def _compile_and_share_module(
     )
     global_client.key_value_set_bytes(cache_key, serialized_executable)
   else:
+    logger.debug("Waiting for module: %s from process %d", module_name,
+                 first_process_id)
     serialized_executable = global_client.blocking_key_value_get_bytes(
         cache_key, share_timeout
     )
@@ -507,6 +614,20 @@ def _compile_and_write_cache(
   )
   return executable
 
+def _is_executable_in_cache(backend, cache_key) -> bool:
+  """Checks if executable is presented in cache on a given key
+  """
+  try:
+    return compilation_cache.is_executable_in_cache(backend, cache_key)
+  except Exception as ex:
+    if config.raise_persistent_cache_errors.value:
+      raise
+    warnings.warn(
+        f"Error reading persistent compilation cache entry for "
+        f"'{cache_key}': {type(ex).__name__}: {ex}")
+    return False
+
+
 def _cache_read(
     module_name: str, cache_key: str, compile_options: xc.CompileOptions,
     backend: xc.Client
