@@ -40,10 +40,10 @@ from jax._src.interpreters import partial_eval as pe
 from jax._src.interpreters import xla
 from jax._src.interpreters.batching import not_mapped
 from jax._src.lax import lax
-from jax._src.tree_util import (tree_flatten, tree_unflatten, tree_map,
-                                treedef_is_leaf, treedef_tuple,
-                                register_pytree_node_class, tree_leaves,
-                                tree_flatten_with_path, keystr)
+from jax._src.tree_util import (
+    tree_flatten, tree_unflatten, tree_map, treedef_is_leaf, treedef_tuple,
+    register_pytree_node_class, tree_leaves, tree_flatten_with_path, keystr,
+    treedef_children)
 from jax._src.util import (cache, safe_zip, safe_map, split_list, Unhashable,
                            unzip2)
 
@@ -252,7 +252,7 @@ class custom_jvp(Generic[ReturnValue]):
       static_args = [args[i] for i in self.nondiff_argnums]
       jvp = _add_args(lu.wrap_init(self.jvp), static_args)
     else:
-      f_, dyn_args = lu.wrap_init(self.fun), args  # type: ignore
+      f_, dyn_args = lu.wrap_init(self.fun), args
       jvp = lu.wrap_init(self.jvp)
     args_flat, in_tree = tree_flatten(dyn_args)
     flat_fun, out_type1 = _flatten_fun_nokwargs(f_, in_tree)
@@ -359,9 +359,9 @@ class CustomJVPCallPrimitive(core.Primitive):
         fun, self, top_trace and top_trace.level, False)
     jvp, env_trace_todo2 = process_env_traces(
         jvp, self, top_trace and top_trace.level, True)
-    tracers = map(top_trace.full_raise, args)  # type: ignore
-    outs = top_trace.process_custom_jvp_call(self, fun, jvp, tracers,  # type: ignore
-                                             symbolic_zeros=symbolic_zeros)  # type: ignore
+    tracers = map(top_trace.full_raise, args)
+    outs = top_trace.process_custom_jvp_call(self, fun, jvp, tracers,
+                                             symbolic_zeros=symbolic_zeros)
     _, env_trace_todo = lu.merge_linear_aux(env_trace_todo1, env_trace_todo2)
     return core.apply_todos(env_trace_todo, map(core.full_lower, outs))
 
@@ -729,6 +729,8 @@ def _flatten_bwd(in_tree, in_avals, out_trees, *args):
   py_res = tree_unflatten(res_tree, res)
   py_cts_out = tree_unflatten(out_tree, cts_out)
   py_cts_in = yield (py_res, py_cts_out), {}
+  if isinstance(py_cts_in, list) and len(py_cts_in) == len(treedef_children(in_tree)):
+    py_cts_in = tuple(py_cts_in)
   # For each None in py_cts_in, indicating an argument for which the rule
   # produces no cotangent, we replace it with a pytree with the structure of the
   # corresponding subtree of in_tree and with leaves of a non-pytree sentinel
@@ -764,7 +766,8 @@ def _flatten_bwd(in_tree, in_avals, out_trees, *args):
       if not core.typecompat(a.at_least_vspace(), a_ := ct.aval):
         msg = ("Custom VJP bwd rule produced a SymbolicZero with a shape/dtype "
                "that does not match the corresponding input tangent shape/dtype: "
-               f"the SymbolicZero had shape/dtype {a_.str_short()} while the "
+               f"at output{keystr(kp)} the SymbolicZero had shape/dtype "
+               f"{a_.str_short()} while the "
                f"corresponding input had shape/dtype {a.str_short()}. "
                "Consider just returning a None here instead of a SymbolicZero "
                "object.")
@@ -805,7 +808,7 @@ class CustomVJPCallPrimitive(core.CallPrimitive):
         fun, self, top_trace and top_trace.level, False)
     fwd, env_trace_todo2 = process_env_traces_fwd(
       fwd, top_trace and top_trace.level, out_trees)
-    tracers = map(top_trace.full_raise, args)  # type: ignore
+    tracers = map(top_trace.full_raise, args)
     bwd_ = lambda *args: bwd(*args)
     outs = top_trace.process_custom_vjp_call(self, fun, fwd, bwd_, tracers,
                                              out_trees=out_trees,
