@@ -48,8 +48,8 @@ from jax._src.monitoring import record_event_duration_secs
 from jax._src.partition_spec import PartitionSpec
 from jax._src.sharding import Sharding
 from jax._src.sharding_impls import (
-    PmapSharding, SingleDeviceSharding, NamedSharding, XLACompatibleSharding,
-    GSPMDSharding, TransferToMemoryKind)
+    SingleDeviceSharding, NamedSharding,
+    GSPMDSharding, TransferToMemoryKind, is_single_device_sharding)
 from jax._src.layout import Layout, DeviceLocalLayout
 
 
@@ -163,12 +163,6 @@ def wait_for_tokens():
   runtime_tokens.block_until_ready()
 
 
-def is_single_device_sharding(sharding: Sharding) -> bool:
-  # Special case PmapSharding here because PmapSharding maps away an axis
-  # and needs to be handled separately.test_pjit_single_device_sharding_add
-  return len(sharding.device_set) == 1 and not isinstance(sharding, PmapSharding)
-
-
 @contextlib.contextmanager
 def log_elapsed_time(fmt: str, fun_name: str, event: str | None = None):
   if _on_exit:
@@ -226,7 +220,7 @@ class SourceInfo(NamedTuple):
 
 def jaxpr_shardings(
     jaxpr: core.Jaxpr,
-) -> Iterator[tuple[XLACompatibleSharding, SourceInfo]]:
+) -> Iterator[tuple[Sharding, SourceInfo]]:
   from jax._src import pjit
   from jax.experimental import shard_map
 
@@ -247,7 +241,7 @@ def jaxpr_shardings(
                   for names in [*eqn.params['in_names'], *eqn.params['out_names']])
     elif eqn.primitive is device_put_p:
       s = eqn.params['device']
-      if isinstance(s, XLACompatibleSharding) and s.memory_kind is not None:
+      if isinstance(s, Sharding) and s.memory_kind is not None:
         source_info = SourceInfo(eqn.source_info, eqn.primitive.name)
         yield (s, source_info)
   for subjaxpr in core.subjaxprs(jaxpr):
@@ -394,13 +388,13 @@ def _device_put_sharding_impl(x, aval, device):
     s = device
     if getattr(x, 'sharding', None) == s and getattr(x, '_committed', False):
       return x
-    if (not s.is_fully_addressable and  # type: ignore
+    if (not s.is_fully_addressable and
         isinstance(x, array.ArrayImpl) and not x.is_fully_addressable):
       # This has to be XLACompatible because _mcjax_reshard will run a
       # XLA computation.
-      assert isinstance(s, XLACompatibleSharding)
+      assert isinstance(s, Sharding)
       return _mcjax_reshard(x, s)
-    if not s.is_fully_addressable:  # type: ignore
+    if not s.is_fully_addressable:
       # TODO(yashkatariya,mattjj): Link to a doc about McJAX and jax.Array.
       raise ValueError(
           "device_put's second argument must be a Device or a Sharding which"
@@ -473,14 +467,14 @@ ad.deflinear2(device_put_p, device_put_transpose_rule)
 batching.defvectorized(device_put_p)
 
 def _tpu_gpu_device_put_lowering(ctx, x, *, device, src):
-  if (isinstance(device, (XLACompatibleSharding, TransferToMemoryKind)) and
+  if (isinstance(device, (Sharding, TransferToMemoryKind)) and
       device.memory_kind is not None):
     aval, = ctx.avals_in
     out_aval, = ctx.avals_out
-    x = mlir.wrap_with_memory_kind(x, device.memory_kind, out_aval)
-    if isinstance(device, XLACompatibleSharding):
+    if isinstance(device, Sharding):
       x = mlir.wrap_with_sharding_op(
           ctx, x, out_aval, device._to_xla_hlo_sharding(aval.ndim).to_proto())
+    x = mlir.wrap_with_memory_kind(x, device.memory_kind, out_aval)
     return [x]
   return [x]
 mlir.register_lowering(
@@ -490,10 +484,16 @@ mlir.register_lowering(
 
 
 def _common_device_put_lowering(ctx, x, *, device, src):
-  if (isinstance(device, (XLACompatibleSharding, TransferToMemoryKind)) and
+  if (isinstance(device, (Sharding, TransferToMemoryKind)) and
       device.memory_kind is not None):
     raise NotImplementedError(
         "Passing memory_kind to device_put via Shardings is not supported on"
         f" platforms {ctx.module_context.platforms}")
   return [x]
 mlir.register_lowering(device_put_p, _common_device_put_lowering)
+
+def _propagate_mem_kind_dp(xm, device=None, src=None):
+  if isinstance(device, (Sharding, TransferToMemoryKind)):
+    return device.memory_kind
+  return None
+pxla.memory_kind_propagate_rule[device_put_p] = _propagate_mem_kind_dp
