@@ -77,7 +77,7 @@ class State:
       raise ValueError('coordinator_bind_address should be defined.')
 
     if local_device_ids:
-      visible_devices = ','.join(str(x) for x in local_device_ids) # type: ignore[union-attr]
+      visible_devices = ','.join(str(x) for x in local_device_ids)
       logger.info('JAX distributed initialized with visible devices: %s', visible_devices)
       config.update("jax_cuda_visible_devices", visible_devices)
       config.update("jax_rocm_visible_devices", visible_devices)
@@ -87,7 +87,9 @@ class State:
     if process_id == 0:
       if self.service is not None:
         raise RuntimeError('distributed.initialize should only be called once.')
-      logger.info('Starting JAX distributed service on %s', coordinator_address)
+      logger.info(
+          'Starting JAX distributed service on %s', coordinator_bind_address
+      )
       self.service = xla_extension.get_distributed_runtime_service(
           coordinator_bind_address, num_processes)
 
@@ -174,7 +176,8 @@ def initialize(coordinator_address: str | None = None,
       have the coordinator service listen on one address/interface.
 
   Raises:
-    RuntimeError: If :func:`~jax.distributed.initialize` is called more than once.
+    RuntimeError: If :func:`~jax.distributed.initialize` is called more than once
+      or if called after the backend is already initialized.
 
   Example:
 
