@@ -24,10 +24,12 @@ import jaxlib.mlir.dialects.scf as scf
 import jaxlib.mlir.dialects.sparse_tensor as sparse_tensor
 import jaxlib.mlir.dialects.vector as vector
 try:
-  import jaxlib.mlir.dialects.gpu as gpu  # type: ignore
-  import jaxlib.mlir.dialects.nvgpu as nvgpu  # type: ignore
-  import jaxlib.mlir.dialects.nvvm as nvvm  # type: ignore
-  import jaxlib.mlir.dialects.llvm as llvm  # type: ignore
+  # pytype: disable=import-error
+  import jaxlib.mlir.dialects.gpu as gpu
+  import jaxlib.mlir.dialects.nvgpu as nvgpu
+  import jaxlib.mlir.dialects.nvvm as nvvm
+  import jaxlib.mlir.dialects.llvm as llvm
+  # pytype: enable=import-error
 except ImportError:
   pass
 
