@@ -512,6 +512,10 @@ def log_softmax(x: ArrayLike,
   Returns:
     An array.
 
+  Note:
+    If any input values are ``+inf``, the result will be all ``NaN``: this reflects the
+    fact that ``inf / inf`` is not well-defined in the context of floating-point math.
+
   See also:
     :func:`softmax`
   """
@@ -557,6 +561,10 @@ def softmax(x: ArrayLike,
   Returns:
     An array.
 
+  Note:
+    If any input values are ``+inf``, the result will be all ``NaN``: this reflects the
+    fact that ``inf / inf`` is not well-defined in the context of floating-point math.
+
   See also:
     :func:`log_softmax`
   """
@@ -569,7 +577,7 @@ def softmax(x: ArrayLike,
     # mypy is confused by the `functools.partial` application in the definition
     # of `_softmax` and incorrectly concludes that `_softmax` returns
     # `ReturnValue` -- the unsubstituted type parameter of `custom_jvp`.
-    return _softmax(x, axis, where)  # type: ignore[return-value]
+    return _softmax(x, axis, where)
   else:
     return _softmax_deprecated(x, axis, where)
 
