@@ -218,9 +218,37 @@ atanh = _one_to_one_unop(getattr(np, "atanh", np.arctanh), lax.atanh, True)
 atan2 = _one_to_one_binop(getattr(np, "atan2", np.arctan2), lax.atan2, True)
 
 
-@implements(getattr(np, 'bitwise_count', None), module='numpy')
 @jit
 def bitwise_count(x: ArrayLike, /) -> Array:
+  r"""Counts the number of 1 bits in the binary representation of the absolute value
+  of each element of ``x``.
+
+  LAX-backend implementation of :func:`numpy.bitwise_count`.
+
+  Args:
+    x: Input array, only accepts integer subtypes
+
+  Returns:
+    An array-like object containing the binary 1 bit counts of the absolute value of
+    each element in ``x``, with the same shape as ``x`` of dtype uint8.
+
+  Example:
+    >>> x1 = jnp.array([64, 32, 31, 20])
+    >>> # 64 = 0b1000000, 32 = 0b100000, 31 = 0b11111, 20 = 0b10100
+    >>> jnp.bitwise_count(x1)
+    Array([1, 1, 5, 2], dtype=uint8)
+
+    >>> x2 = jnp.array([-16, -7, 7])
+    >>> # |-16| = 0b10000, |-7| = 0b111, 7 = 0b111
+    >>> jnp.bitwise_count(x2)
+    Array([1, 3, 3], dtype=uint8)
+
+    >>> x3 = jnp.array([[2, -7],[-9, 7]])
+    >>> # 2 = 0b10, |-7| = 0b111, |-9| = 0b1001, 7 = 0b111
+    >>> jnp.bitwise_count(x3)
+    Array([[1, 3],
+           [2, 3]], dtype=uint8)
+  """
   x, = promote_args_numeric("bitwise_count", x)
   # Following numpy we take the absolute value and return uint8.
   return lax.population_count(abs(x)).astype('uint8')
@@ -241,13 +269,46 @@ def bitwise_right_shift(x1: ArrayLike, x2: ArrayLike, /) -> Array:
     np.issubdtype(x1.dtype, np.unsignedinteger) else lax.shift_right_arithmetic
   return lax_fn(x1, x2)
 
-@implements(np.absolute, module='numpy')
+
 @partial(jit, inline=True)
 def absolute(x: ArrayLike, /) -> Array:
+  r"""Calculate the absolute value element-wise.
+
+  LAX-backend implementation of :func:`numpy.absolute`.
+
+  This is the same function as :func:`jax.numpy.abs`.
+
+  Args:
+    x: Input array
+
+  Returns:
+    An array-like object containing the absolute value of each element in ``x``,
+    with the same shape as ``x``. For complex valued input, :math:`a + ib`,
+    the absolute value is :math:`\sqrt{a^2+b^2}`.
+
+  Example:
+    >>> x1 = jnp.array([5, -2, 0, 12])
+    >>> jnp.absolute(x1)
+    Array([ 5,  2,  0, 12], dtype=int32)
+
+    >>> x2 = jnp.array([[ 8, -3, 1],[ 0, 9, -6]])
+    >>> jnp.absolute(x2)
+    Array([[8, 3, 1],
+           [0, 9, 6]], dtype=int32)
+
+    >>> x3 = jnp.array([8 + 15j, 3 - 4j, -5 + 0j])
+    >>> jnp.absolute(x3)
+    Array([17.,  5.,  5.], dtype=float32)
+  """
   check_arraylike('absolute', x)
   dt = dtypes.dtype(x)
   return lax.asarray(x) if dt == np.bool_ or dtypes.issubdtype(dt, np.unsignedinteger) else lax.abs(x)
-abs = implements(np.abs, module='numpy')(absolute)
+
+
+@partial(jit, inline=True)
+def abs(x: ArrayLike, /) -> Array:
+  """Alias of :func:`jax.numpy.absolute`."""
+  return absolute(x)
 
 
 @implements(np.rint, module='numpy')
