@@ -136,6 +136,10 @@ def _bcast_to(a: ir.Value, shape: tuple[int, ...]) -> ir.Value:
     a_type = ir.RankedTensorType(a.type)
     if a_type.shape == [*shape]:
       return a
+    if a_type.rank != len(shape) or not all(
+        a_type.shape[i] in (dim, 1) for i, dim in enumerate(shape)
+    ):
+      raise ValueError(f"Cannot broadcast from {a_type.shape} to {[*shape]}")
     return tt_dialect.broadcast(
         ir.RankedTensorType.get(shape, a_type.element_type, a_type.encoding), a
     )
@@ -382,20 +386,17 @@ def _program_id(axis: int) -> ir.Value:
   return tt_dialect.get_program_id(axis)
 
 
+@register_lowering(primitives.program_id_p)
 def _program_id_lowering_rule(ctx: LoweringRuleContext, *, axis):
   return ctx.context.program_ids[axis]
 
 
-triton_lowering_rules[primitives.program_id_p] = _program_id_lowering_rule
-
-
+@register_lowering(primitives.num_programs_p)
 def _num_programs_lowering_rule(ctx: LoweringRuleContext, *, axis):
   if axis not in range(3):
     raise ValueError(f"axis must be in [0, 3), but got: {axis}")
   return tt_dialect.get_num_programs(axis)
 
-triton_lowering_rules[primitives.num_programs_p] = _num_programs_lowering_rule
-
 
 def _atomic_rmw(
     op: tt_dialect.RMWOp,
@@ -418,6 +419,7 @@ def _atomic_rmw(
   )
 
 
+@register_lowering(primitives.atomic_rmw_p)
 def _atomic_lowering_rule(
     ctx: LoweringRuleContext,
     *args_flat,
@@ -457,9 +459,7 @@ def _atomic_lowering_rule(
   return _atomic_rmw(op, ptr, val, mask=mask)
 
 
-triton_lowering_rules[primitives.atomic_rmw_p] = _atomic_lowering_rule
-
-
+@register_lowering(primitives.atomic_cas_p)
 def _atomic_cas_lowering_rule(ctx: LoweringRuleContext, ptr, cmp, val):
   _, cmp_aval, val_aval = ctx.avals_in
   if ir.RankedTensorType.isinstance(ptr.type):
@@ -480,9 +480,6 @@ def _atomic_cas_lowering_rule(ctx: LoweringRuleContext, ptr, cmp, val):
   )
 
 
-triton_lowering_rules[primitives.atomic_cas_p] = _atomic_cas_lowering_rule
-
-
 def _associative_scan_lowering(body, ctx: LoweringRuleContext, args, axes):
   flat_args = tree_util.tree_leaves(args)
   (axis,) = axes
@@ -515,6 +512,7 @@ def _associative_scan_lowering(body, ctx: LoweringRuleContext, args, axes):
   return list(scan_op.result)
 
 
+@register_lowering(lax.cumsum_p)
 def _cumsum_lowering_rule(
     ctx: LoweringRuleContext, x, *, axis: int, reverse: bool
 ):
@@ -523,17 +521,12 @@ def _cumsum_lowering_rule(
   return _associative_scan_lowering(jnp.add, ctx, x, (axis,))[0]
 
 
-triton_lowering_rules[lax.cumsum_p] = _cumsum_lowering_rule
-
-
+@register_lowering(lax.not_p)
 def _not_lowering_rule(ctx: LoweringRuleContext, x):
   [x_aval] = ctx.avals_in
   return arith_dialect.xori(x, _full(x.type, ~x_aval.dtype.type(0)))
 
 
-triton_lowering_rules[lax.not_p] = _not_lowering_rule
-
-
 @dataclasses.dataclass(frozen=True)
 class _Extern:
   arg_types: Sequence[str]
@@ -606,6 +599,7 @@ _abs_dispatch_table = _make_dispatch_table(
 )
 
 
+@register_lowering(lax.abs_p)
 def _abs_lowering_rule(ctx: LoweringRuleContext, x):
   try:
     return _abs_dispatch_table(ctx, x)
@@ -619,9 +613,6 @@ def _abs_lowering_rule(ctx: LoweringRuleContext, x):
       raise e from None
 
 
-triton_lowering_rules[lax.abs_p] = _abs_lowering_rule
-
-
 triton_lowering_rules.update({
     lax.neg_p: lambda ctx, x: _minus(x),
     lax.ceil_p: _make_dispatch_table(
@@ -1019,6 +1010,22 @@ for prim, fn in _JAX_TO_TRITON_SIGNED_BINARY.items():
   triton_lowering_rules[prim] = signed_rule
 
 
+@register_lowering(primitives.debug_print_p)
+def debug_print_lowering_rule(
+    ctx: LoweringRuleContext,
+    *args: ir.Value,
+    fmt: str,
+    has_placeholders: bool,
+):
+  if has_placeholders:
+    raise ValueError(
+        "pl.debug_print() does not support placeholders when lowering to Triton"
+    )
+
+  tt_dialect.print_(f" {fmt} ", hex=False, args=args)
+  return ()
+
+
 def _set_attr(v: ir.Value, name: str, attr: ir.Attribute) -> None:
   if not ir.BlockArgument.isinstance(v):
     v.owner.attributes[name] = attr
@@ -1034,6 +1041,7 @@ def _set_attr(v: ir.Value, name: str, attr: ir.Attribute) -> None:
     op.attributes[name] = attr
 
 
+@register_lowering(primitives.multiple_of_p)
 def _multiple_of_rule(ctx: LoweringRuleContext, x, values: Sequence[int]):
   [x_aval] = ctx.avals_in
   assert max(1, len(x_aval.shape)) == len(values)
@@ -1045,9 +1053,7 @@ def _multiple_of_rule(ctx: LoweringRuleContext, x, values: Sequence[int]):
   return x
 
 
-triton_lowering_rules[primitives.multiple_of_p] = _multiple_of_rule
-
-
+@register_lowering(primitives.max_contiguous_p)
 def _max_contiguous_rule(ctx: LoweringRuleContext, x, values: Sequence[int]):
   [x_aval] = ctx.avals_in
   assert len(x_aval.shape) == len(values)
@@ -1059,17 +1065,12 @@ def _max_contiguous_rule(ctx: LoweringRuleContext, x, values: Sequence[int]):
   return x
 
 
-triton_lowering_rules[primitives.max_contiguous_p] = _max_contiguous_rule
-
-
+@register_lowering(sp.broadcast_to_p)
 def _broadcast_to_rule(ctx: LoweringRuleContext, x, shape: Sequence[int]):
   (x_aval,) = ctx.avals_in
   return _bcast_to(_ensure_ir_value(x, x_aval), shape)
 
 
-triton_lowering_rules[sp.broadcast_to_p] = _broadcast_to_rule
-
-
 def _integer_pow(a, *, y):
   if y == 2:
     return a * a
@@ -1105,6 +1106,7 @@ for prim, fn in _JAX_FN_MAPPING.items():
   triton_lowering_rules[prim] = lower_fun(fn, multiple_results=False)
 
 
+@register_lowering(lax.min_p)
 def _min_lowering_rule(ctx: LoweringRuleContext, x, y):
   # TODO(slebedev): Consider allowing customizing nan behavior.
   x_aval, y_aval = ctx.avals_in
@@ -1122,9 +1124,7 @@ def _min_lowering_rule(ctx: LoweringRuleContext, x, y):
     return arith_dialect.minui(x, y)
 
 
-triton_lowering_rules[lax.min_p] = _min_lowering_rule
-
-
+@register_lowering(lax.max_p)
 def _max_lowering_rule(ctx: LoweringRuleContext, x, y):
   # TODO(slebedev): Consider allowing customizing nan behavior.
   x_aval, y_aval = ctx.avals_in
@@ -1142,9 +1142,7 @@ def _max_lowering_rule(ctx: LoweringRuleContext, x, y):
     return arith_dialect.maxui(x, y)
 
 
-triton_lowering_rules[lax.max_p] = _max_lowering_rule
-
-
+@register_lowering(lax.div_p)
 def _div_lowering_rule(ctx: LoweringRuleContext, x, y):
   x_aval, y_aval = ctx.avals_in
   x, y = _bcast(x, y, *ctx.avals_in, *ctx.avals_out)
@@ -1158,9 +1156,7 @@ def _div_lowering_rule(ctx: LoweringRuleContext, x, y):
   return _floordiv(x, y, signed=signed)
 
 
-triton_lowering_rules[lax.div_p] = _div_lowering_rule
-
-
+@register_lowering(lax.sign_p)
 def _sign_lowering_rule(ctx: LoweringRuleContext, x):
   [x_aval] = ctx.avals_in
   signed = jnp.issubdtype(x_aval.dtype, jnp.signedinteger)
@@ -1171,9 +1167,7 @@ def _sign_lowering_rule(ctx: LoweringRuleContext, x):
   )
 
 
-triton_lowering_rules[lax.sign_p] = _sign_lowering_rule
-
-
+@register_lowering(lax.iota_p)
 def _iota_lowering_rule(ctx: LoweringRuleContext, *, dtype, shape, dimension):
   iota = _make_range(0, shape[dimension])
   iota = _cast(iota, jnp.int32, dtype)
@@ -1183,9 +1177,6 @@ def _iota_lowering_rule(ctx: LoweringRuleContext, *, dtype, shape, dimension):
   return _bcast_to(iota, shape)
 
 
-triton_lowering_rules[lax.iota_p] = _iota_lowering_rule
-
-
 def _element_type(t: ir.Type) -> ir.Type:
   if ir.RankedTensorType.isinstance(t):
     return ir.RankedTensorType(t).element_type
@@ -1382,6 +1373,7 @@ def _ir_cast(src: ir.Value, dst_type: ir.Type, *, signed: bool) -> ir.Value:
   raise NotImplementedError(f"cannot cast {src} to {dst_type}")
 
 
+@register_lowering(lax.convert_element_type_p)
 def _convert_element_type_lowering_rule(
     ctx: LoweringRuleContext, x, *, new_dtype, weak_type
 ):
@@ -1392,11 +1384,7 @@ def _convert_element_type_lowering_rule(
   return _cast(x, x_aval.dtype, new_dtype)
 
 
-triton_lowering_rules[lax.convert_element_type_p] = (
-    _convert_element_type_lowering_rule
-)
-
-
+@register_lowering(lax.select_n_p)
 def select_n_lowering_rule(ctx: LoweringRuleContext, pred, x, y):
   pred_aval, a_aval, b_aval = ctx.avals_in
   [out_aval] = ctx.avals_out
@@ -1405,9 +1393,7 @@ def select_n_lowering_rule(ctx: LoweringRuleContext, pred, x, y):
   return arith_dialect.select(pred, y, x)
 
 
-triton_lowering_rules[lax.select_n_p] = select_n_lowering_rule
-
-
+@register_lowering(lax.broadcast_in_dim_p)
 def _broadcast_in_dim_lowering_rule(
     ctx: LoweringRuleContext, x, *, broadcast_dimensions, shape
 ):
@@ -1420,19 +1406,12 @@ def _broadcast_in_dim_lowering_rule(
   return _bcast_to(x, shape)
 
 
-triton_lowering_rules[jax.lax.broadcast_in_dim_p] = (
-    _broadcast_in_dim_lowering_rule
-)
-
-
+@register_lowering(lax.squeeze_p)
 def _squeeze_lowering_rule(ctx: LoweringRuleContext, a, *, dimensions):
   del dimensions
   return _reshape_lowering_rule(ctx, a, new_sizes=None, dimensions=None)
 
 
-triton_lowering_rules[lax.squeeze_p] = _squeeze_lowering_rule
-
-
 def _reshape(x: ir.Value, shape: Sequence[int]) -> ir.Value:
   if not shape:
     raise ValueError("cannot reshape to an empty shape")
@@ -1444,6 +1423,7 @@ def _reshape(x: ir.Value, shape: Sequence[int]) -> ir.Value:
   )
 
 
+@register_lowering(lax.reshape_p)
 def _reshape_lowering_rule(
     ctx: LoweringRuleContext, a, *, new_sizes, dimensions
 ):
@@ -1487,9 +1467,6 @@ def _reshape_lowering_rule(
   return a
 
 
-triton_lowering_rules[jax.lax.reshape_p] = _reshape_lowering_rule
-
-
 def _compute_pointers_from_indices(
     root_ptr: ir.Value,
     block_info: BlockInfo | None,
@@ -1528,14 +1505,22 @@ def _compute_pointers_from_indices(
     else:
       index = next(indexer_iter)
     if isinstance(index, primitives.Slice):
-      # Handle slices with static and dynamic indices and static sizes
-      if isinstance(index.start, int):
-        ptr_dim_offset = _make_range(index.start, index.start + index.size)
-      else:
+      if index.is_dynamic_start:
+        # Compute the offset as start + range(0, size).
         ptr_dim_offset = _add(
             _bcast_to(index.start, [index.size]),
             _ir_cast(_make_range(0, index.size), index.start.type, signed=False),
         )
+      elif index.stride > 1:
+        # Compute the offset as start + range(0, size) * stride.
+        iota = _make_range(0, index.size)
+        ptr_dim_offset = _add(
+            _bcast_to(_i32_constant(index.start), [index.size]),
+            _mul(iota, _full(iota.type, index.stride)),
+        )
+      else:
+        ptr_dim_offset = _make_range(index.start, index.start + index.size)
+
       # We need to add broadcastable dimensions for the advanced int indexing
       # and for previous slices
       num_left_expand_dims = len(int_indexer_shape) + other_shape_idx
@@ -1589,6 +1574,7 @@ def _compute_pointers_from_indices(
   )
 
 
+@register_lowering(sp.get_p)
 def _get_lowering_rule(ctx: LoweringRuleContext, ptr, *idx, tree):
   indexers = tree_util.tree_unflatten(tree, idx)
   if not tt_dialect.PointerType.isinstance(ptr.type):
@@ -1608,9 +1594,6 @@ def _get_lowering_rule(ctx: LoweringRuleContext, ptr, *idx, tree):
   )
 
 
-triton_lowering_rules[sp.get_p] = _get_lowering_rule
-
-
 _STR_TO_EVICTION_POLICY = {str(e): e for e in tt_dialect.EvictionPolicy}
 _STR_TO_CACHE_MODIFIER = {str(c): c for c in tt_dialect.CacheModifier}
 
@@ -1685,6 +1668,7 @@ def _load(
   )
 
 
+@register_lowering(primitives.load_p)
 def _masked_load_lowering_rule(
     ctx: LoweringRuleContext,
     *args_flat,
@@ -1720,9 +1704,7 @@ def _masked_load_lowering_rule(
   )
 
 
-triton_lowering_rules[primitives.load_p] = _masked_load_lowering_rule
-
-
+@register_lowering(sp.swap_p)
 def _swap_lowering_rule(ctx: LoweringRuleContext, ptr, value, *idx, tree):
   indexers = tree_util.tree_unflatten(tree, idx)
   if not tt_dialect.PointerType.isinstance(ptr.type):
@@ -1737,9 +1719,6 @@ def _swap_lowering_rule(ctx: LoweringRuleContext, ptr, value, *idx, tree):
   )
 
 
-triton_lowering_rules[sp.swap_p] = _swap_lowering_rule
-
-
 def _store(
     ptr: ir.Value,
     value: ir.Value,
@@ -1794,6 +1773,7 @@ def _store(
   )
 
 
+@register_lowering(primitives.swap_p)
 def _masked_swap_lowering_rule(
     ctx: LoweringRuleContext, *args_flat, args_tree, eviction_policy
 ):
@@ -1818,9 +1798,7 @@ def _masked_swap_lowering_rule(
   return old_value
 
 
-triton_lowering_rules[primitives.swap_p] = _masked_swap_lowering_rule
-
-
+@register_lowering(sp.addupdate_p)
 def _addupdate_lowering_rule(ctx: LoweringRuleContext, ptr, value, *idx, tree):
   indexers = tree_util.tree_unflatten(tree, idx)
   if not tt_dialect.PointerType.isinstance(ptr.type):
@@ -1842,16 +1820,11 @@ def _addupdate_lowering_rule(ctx: LoweringRuleContext, ptr, value, *idx, tree):
   return []
 
 
-triton_lowering_rules[sp.addupdate_p] = _addupdate_lowering_rule
-
-
+@register_lowering(lax.transpose_p)
 def _transpose_lowering(ctx: LoweringRuleContext, x, *, permutation):
   return tt_dialect.trans(x, permutation)
 
 
-triton_lowering_rules[lax.transpose_p] = _transpose_lowering
-
-
 def _check_dot_operands(
     x_type: ir.RankedTensorType, y_type: ir.RankedTensorType, options: Any
 ):
@@ -1930,6 +1903,7 @@ def _dot(
 _TF32_PRECISIONS = (lax.Precision.HIGH, lax.Precision.DEFAULT)
 
 
+@register_lowering(lax.dot_general_p)
 def _dot_general_lowering(
     ctx: LoweringRuleContext,
     a,
@@ -1971,9 +1945,6 @@ def _dot_general_lowering(
   )
 
 
-triton_lowering_rules[lax.dot_general_p] = _dot_general_lowering
-
-
 def _reduction_lowering(body, ctx: LoweringRuleContext, a, axes):
   flat_args = tree_util.tree_leaves(a)
   (axis,) = axes
@@ -2086,6 +2057,7 @@ triton_lowering_rules[lax.argmin_p] = functools.partial(
 )
 
 
+@register_lowering(pjit.pjit_p)
 def _pjit_lowering_rule(ctx: LoweringRuleContext, *args, jaxpr, **_):
   if jaxpr.consts:
     raise NotImplementedError
@@ -2094,9 +2066,8 @@ def _pjit_lowering_rule(ctx: LoweringRuleContext, *args, jaxpr, **_):
   )
 
 
-triton_lowering_rules[pjit.pjit_p] = _pjit_lowering_rule
-
-
+@register_lowering(jax_core.closed_call_p)
+@register_lowering(custom_derivatives.custom_jvp_call_p)
 def _closed_call_lowering_rule(
     ctx: LoweringRuleContext, *args, call_jaxpr, **_
 ):
@@ -2106,17 +2077,11 @@ def _closed_call_lowering_rule(
   return lower_jaxpr_to_triton_ir(ctx.context, jaxpr, ctx.block_infos, *args)
 
 
-triton_lowering_rules[jax_core.closed_call_p] = _closed_call_lowering_rule
-triton_lowering_rules[custom_derivatives.custom_jvp_call_p] = (
-    _closed_call_lowering_rule
-)
-
-
+@register_lowering(ad_checkpoint.remat_p)
 def _remat_lowering_rule(ctx: LoweringRuleContext, *args, jaxpr, **_):
   return lower_jaxpr_to_triton_ir(ctx.context, jaxpr, ctx.block_infos, *args)
 
 
-triton_lowering_rules[ad_checkpoint.remat_p] = _remat_lowering_rule
 triton_lowering_rules[ad_util.stop_gradient_p] = lambda _, x: x
 
 
@@ -2130,6 +2095,7 @@ def _is_read_only(ref_effects) -> bool:
   return isinstance(eff, state.ReadEffect)
 
 
+@register_lowering(for_loop.for_p)
 def _for_lowering_rule(
     ctx: LoweringRuleContext,
     *args,
@@ -2183,9 +2149,6 @@ def _for_lowering_rule(
   return merge_lists(is_loop_arg, non_loop_args, list(for_op.results_))
 
 
-triton_lowering_rules[for_loop.for_p] = _for_lowering_rule
-
-
 def _lower_jaxpr_to_for_loop(
     ctx: LoweringRuleContext,
     jaxpr: jax_core.Jaxpr,
@@ -2222,6 +2185,7 @@ def _lower_jaxpr_to_for_loop(
   return list(for_op.results_)
 
 
+@register_lowering(lax.scan_p)
 def _scan_lowering_rule(
     ctx: LoweringRuleContext,
     *args,
@@ -2271,9 +2235,6 @@ def _scan_lowering_rule(
   return for_out
 
 
-triton_lowering_rules[lax.scan_p] = _scan_lowering_rule
-
-
 def _maybe_pattern_match_fori_loop(
     ctx: LoweringRuleContext,
     *args,
@@ -2354,6 +2315,7 @@ def _maybe_pattern_match_fori_loop(
   return [ub, ub, *for_out]
 
 
+@register_lowering(lax.while_p)
 def _while_lowering_rule(
     ctx: LoweringRuleContext,
     *args,
@@ -2423,9 +2385,7 @@ def _while_lowering_rule(
   return all_out[cond_nconsts + body_nconsts :]
 
 
-triton_lowering_rules[lax.while_p] = _while_lowering_rule
-
-
+@register_lowering(lax.cond_p)
 def _cond_lowering_rule(
     ctx: LoweringRuleContext,
     index,
@@ -2474,9 +2434,6 @@ def _cond_lowering_rule(
   return list(if_op.results_)
 
 
-triton_lowering_rules[lax.cond_p] = _cond_lowering_rule
-
-
 def _ensure_ir_value(x: object, aval: jax_core.ShapedArray) -> ir.Value:
   if isinstance(x, ir.Value):
     return x
