@@ -20,8 +20,6 @@ XLA. There are also a handful of related casting utilities.
 """
 from __future__ import annotations
 
-from __future__ import annotations
-
 import atexit
 from collections.abc import Mapping
 import dataclasses
@@ -32,28 +30,28 @@ import logging
 import os
 import pkgutil
 import platform as py_platform
-import traceback
 import sys
 import threading
+import traceback
 from typing import Any, Callable, Union
 import warnings
 
 from jax._src import config
 from jax._src import distributed
+from jax._src import hardware_utils
 from jax._src import traceback_util
 from jax._src import util
-from jax._src import hardware_utils
 from jax._src.cloud_tpu_init import maybe_import_libtpu
 from jax._src.lib import cuda_versions
 from jax._src.lib import xla_client
 from jax._src.lib import xla_extension
-from jax._src.lib import jaxlib
+from jax._src.lib import xla_extension_version
 
 logger = logging.getLogger(__name__)
 
 jax_plugins: Any | None
 try:
-  import jax_plugins  # type: ignore
+  import jax_plugins  # pytype: disable=import-error
 except ModuleNotFoundError:
   jax_plugins = None
 except ImportError as e:
@@ -160,7 +158,13 @@ def tpu_client_timer_callback(timer_secs: float) -> xla_client.Client | None:
   t.start()
 
   try:
-    client = xla_client.make_tpu_client(_get_tpu_library_path())
+    if xla_extension_version >= 267:
+      client = xla_client.make_tpu_client( # type: ignore
+          _get_tpu_library_path(),
+          _options_from_jax_configs("tpu"))
+    else:
+      client = xla_client.make_tpu_client(
+          _get_tpu_library_path())
   finally:
     t.cancel()
 
@@ -246,19 +250,19 @@ def make_cpu_client() -> xla_client.Client:
                     DeprecationWarning,
                     )
   if collectives_impl == 'gloo':
-    collectives = xla_client._xla.make_gloo_tcp_collectives(  # type: ignore
+    collectives = xla_client._xla.make_gloo_tcp_collectives(
       distributed_client=distributed.global_state.client,
     )
   elif collectives_impl == 'mpi':
-    collectives = xla_client._xla.make_mpi_collectives()  # type: ignore
-    collectives.Init()  # type: ignore
-    atexit.register(collectives.Finalize)  # type: ignore
+    collectives = xla_client._xla.make_mpi_collectives()
+    collectives.Init()
+    atexit.register(collectives.Finalize)
   elif collectives_impl != 'none':
     collectives_impls = ['none', 'gloo', 'mpi']
     raise RuntimeError(f"Unknown collectives implementation "
                        f"{collectives_impl}. Available implementations are "
                        f"{collectives_impls}.")
-  return xla_client.make_cpu_client(  # type: ignore
+  return xla_client.make_cpu_client(
     asynchronous=_CPU_ENABLE_ASYNC_DISPATCH.value,
     distributed_client=distributed.global_state.client,
     node_id=distributed.global_state.process_id,
@@ -393,19 +397,16 @@ def _check_cuda_versions(raise_on_first_error: bool = False,
   _version_check("cuPTI", cuda_versions.cupti_get_version,
                  cuda_versions.cupti_build_version,
                  min_supported_version=18)
-  # TODO(jakevdp) remove these checks when minimum jaxlib is v0.4.21
-  if hasattr(cuda_versions, "cublas_get_version"):
-    _version_check("cuBLAS", cuda_versions.cublas_get_version,
-                   cuda_versions.cublas_build_version,
-                   # Ignore patch versions.
-                   scale_for_comparison=100,
-                   min_supported_version=120100)
-  if hasattr(cuda_versions, "cusparse_get_version"):
-    _version_check("cuSPARSE", cuda_versions.cusparse_get_version,
-                   cuda_versions.cusparse_build_version,
-                   # Ignore patch versions.
-                   scale_for_comparison=100,
-                   min_supported_version=12100)
+  _version_check("cuBLAS", cuda_versions.cublas_get_version,
+                 cuda_versions.cublas_build_version,
+                 # Ignore patch versions.
+                 scale_for_comparison=100,
+                 min_supported_version=120100)
+  _version_check("cuSPARSE", cuda_versions.cusparse_get_version,
+                 cuda_versions.cusparse_build_version,
+                 # Ignore patch versions.
+                 scale_for_comparison=100,
+                 min_supported_version=12100)
 
   errors = []
   debug_results = []
@@ -447,11 +448,12 @@ def make_gpu_client(
       print('Skipped CUDA versions constraints check due to the '
             'JAX_SKIP_CUDA_CONSTRAINTS_CHECK env var being set.')
 
-    # TODO(micky774): remove this check when minimum jaxlib is v0.4.26
-    if jaxlib.version.__version_info__ >= (0, 4, 26):
-      devices_to_check = (allowed_devices if allowed_devices else
-                          range(cuda_versions.cuda_device_count()))
-      _check_cuda_compute_capability(devices_to_check)
+    devices_to_check = (
+        allowed_devices
+        if allowed_devices
+        else range(cuda_versions.cuda_device_count())
+    )
+    _check_cuda_compute_capability(devices_to_check)
 
   return xla_client.make_gpu_client(
       distributed_client=distributed.global_state.client,
@@ -459,7 +461,7 @@ def make_gpu_client(
       num_nodes=num_nodes,
       platform_name=platform_name,
       allowed_devices=allowed_devices,
-      mock=use_mock_gpu_client,  # type: ignore[call-arg]
+      mock=use_mock_gpu_client,
   )
 
 
@@ -618,16 +620,30 @@ def discover_pjrt_plugins() -> None:
 
 
 def _options_from_jax_configs(plugin_name):
-  if plugin_name != "cuda":
-    return {}
-
   options = {}
-  visible_devices = CUDA_VISIBLE_DEVICES.value
-  if visible_devices != 'all':
-    options['visible_devices'] = [int(x) for x in visible_devices.split(',')]
-  options['enable_mock_nccl'] = _USE_MOCK_GPU_CLIENT.value
-  if options['enable_mock_nccl']:
-    options['num_nodes'] = _MOCK_NUM_GPUS.value
+
+  pjrt_client_options = config.jax_pjrt_client_create_options.value
+  pjrt_client_option_list = []
+  if pjrt_client_options:
+    pjrt_client_option_list = pjrt_client_options.split(";")
+
+  for option in pjrt_client_option_list:
+    option_list = option.split(":")
+    if (len(option_list) != 2):
+      raise RuntimeError(
+          "Multiple ':' separators for option in "
+          f"jax_pjrt_client_create_options: '{option}'. "
+          "Should be in format 'key:value'")
+    options[option_list[0]] = option_list[1]
+
+  if plugin_name == "cuda":
+    visible_devices = CUDA_VISIBLE_DEVICES.value
+    if visible_devices != 'all':
+      options['visible_devices'] = [int(x) for x in visible_devices.split(',')]
+    options['enable_mock_nccl'] = _USE_MOCK_GPU_CLIENT.value
+    if options['enable_mock_nccl']:
+      options['num_nodes'] = _MOCK_NUM_GPUS.value
+
   return options
 
 
@@ -691,7 +707,7 @@ def register_plugin(
       'registering PJRT plugin %s from %s', plugin_name, library_path
   )
   if library_path is not None:
-    c_api = xla_client.load_pjrt_plugin_dynamically(plugin_name, library_path)  # type: ignore
+    c_api = xla_client.load_pjrt_plugin_dynamically(plugin_name, library_path)
     xla_client.profiler.register_plugin_profiler(c_api)
   else:
     assert c_api is not None
@@ -1212,7 +1228,8 @@ def make_pjrt_tpu_topology(topology_name='', **kwargs):
       raise RuntimeError(
           "JAX TPU support not installed; cannot generate TPU topology. See"
           " https://github.com/google/jax#installation")
-    xla_client.load_pjrt_plugin_dynamically("tpu", library_path)
+    c_api = xla_client.load_pjrt_plugin_dynamically("tpu", library_path)
+    xla_client.profiler.register_plugin_profiler(c_api)
   assert xla_client.pjrt_plugin_loaded("tpu")
   if not xla_client.pjrt_plugin_initialized("tpu"):
     xla_client.initialize_pjrt_plugin("tpu")
