@@ -133,10 +133,8 @@ def _custom_partitioning_propagate_user_sharding(user_sharding, shape,
 
 
 def _to_hlo_sharding(sharding, num_dimensions):
-  if not isinstance(sharding, jax.sharding.XLACompatibleSharding):
-    raise ValueError(
-        "Custom Partitioning rules must return XLACompatibleShardings."
-    )
+  if not isinstance(sharding, jax.sharding.Sharding):
+    raise ValueError("Custom Partitioning rules must return Sharding.")
   return sharding._to_xla_hlo_sharding(num_dimensions)
 
 
@@ -182,13 +180,14 @@ def _custom_partitioning_partition(arg_shapes, arg_shardings, result_shape,
         % (repr(closed_jaxpr.out_avals), repr(tiled_results))
     )
   axis_context = sharding_impls.SPMDAxisContext(mesh)
-  module = mlir.build_mlir_module_helper(
-      closed_jaxpr,
-      name="tmp_xla_computation",
-      platforms=module_context.platforms,
-      backend_or_name=module_context.backend_or_name,
-      axis_context=axis_context.extend_manual(frozenset(mesh.axis_names)),
-  )
+  with core.extend_axis_env_nd(mesh.shape.items()):
+    module = mlir.build_mlir_module_helper(
+        closed_jaxpr,
+        name="tmp_xla_computation",
+        platforms=module_context.platforms,
+        backend_or_name=module_context.backend_or_name,
+        axis_context=axis_context.extend_manual(frozenset(mesh.axis_names)),
+    )
   result_sharding = _pack_result_sharding(result_shape, result_shardings)
   return mlir.module_to_bytecode(module), arg_shardings, result_sharding
 
@@ -536,11 +535,11 @@ def _custom_partitioning_lowering_rule(ctx: mlir.LoweringRuleContext, *values,
 mlir.register_lowering(custom_partitioning_p,
                        _custom_partitioning_lowering_rule)
 
-xc.register_custom_call_partitioner(  # pytype: disable=module-attr
+xc.register_custom_call_partitioner(
     _CUSTOM_PARTITIONING_CALL_NAME,
     _custom_partitioning_propagate_user_sharding,
     _custom_partitioning_partition,
-    _custom_partitioning_infer_sharding_from_operands, True)
+    _custom_partitioning_infer_sharding_from_operands, True)  # type: ignore
 xb.register_plugin_callbacks(
     partial(
         xc.register_custom_call_partitioner,
