@@ -13,7 +13,7 @@
 # limitations under the License.
 # ==============================================================================
 
-from jax.experimental.export._export import (
+from jax._src.export._export import (
     minimum_supported_serialization_version,
     maximum_supported_serialization_version,
     Exported,
@@ -22,17 +22,15 @@ from jax.experimental.export._export import (
     call,
     DisabledSafetyCheck,
     default_lowering_platform,
-
-    args_specs,  # TODO: deprecate
 )
-from jax.experimental.export._shape_poly import (
+from jax._src.export.shape_poly import (
     is_symbolic_dim,
     symbolic_shape,
     symbolic_args_specs,
     SymbolicScope,
 )
-from jax.experimental.export._serialization import (
+from jax._src.export.serialization import (
     serialize,
     deserialize,
 )
-from jax.experimental.export import _shape_poly_decision
+from jax._src.export import shape_poly_decision
