@@ -17,12 +17,14 @@ from collections.abc import Callable
 import contextlib
 import ctypes
 import dataclasses
+import functools
+import itertools
 import os
 import pathlib
 import subprocess
 import tempfile
 import time
-from typing import Any, Sequence
+from typing import Any, Generic, Sequence, TypeVar
 
 import jax
 from jax._src import config
@@ -39,7 +41,6 @@ from jaxlib.mlir.dialects import llvm
 from jaxlib.mlir.dialects import memref
 from jaxlib.mlir.dialects import nvgpu
 from jaxlib.mlir.dialects import nvvm
-from jaxlib.mlir.execution_engine import ExecutionEngine
 from jaxlib.mlir.passmanager import PassManager
 import numpy as np
 
@@ -67,34 +68,10 @@ TMA_DESCRIPTOR_ALIGNMENT = 64
 c = mgpu.c  # This is too common to fully qualify.
 
 
-xla_client.register_custom_call_target(
-    "mosaic_gpu",
-    mosaic_gpu_lib._mosaic_gpu_ext._custom_call_capsule(),
-    platform="CUDA",
-)
-mosaic_gpu_lib._mosaic_gpu_ext.register_passes()
-
-
-mosaic_gpu_dump_ptx = config.define_bool_state(
-    name="mosaic_gpu_dump_ptx",
-    default=config.bool_env("MOSAIC_GPU_DUMP_PTX", False),
-    help="If set, prints the kernel PTX",
-)
-mosaic_gpu_dump_ptxas = config.define_bool_state(
-    name="mosaic_gpu_dump_ptxas",
-    default=config.bool_env("MOSAIC_GPU_DUMP_PTXAS", False),
-    help="If set, prints the ptxas verbose output",
-)
-mosaic_gpu_dump_sass = config.define_bool_state(
-    name="mosaic_gpu_dump_sass",
-    default=config.bool_env("MOSAIC_GPU_DUMP_SASS", False),
-    help="If set, prints the kernel SASS",
-)
-mosaic_gpu_print_after_all = config.define_bool_state(
-    name='mosaic_gpu_print_after_all',
-    default=config.bool_env('MOSAIC_GPU_PRINT_AFTER_ALL', False),
-    help="If set, prints the kernel module after every pass",
-)
+RUNTIME_PATH = pathlib.Path(mosaic_gpu_lib._mosaic_gpu_ext.__file__).parent / "libmosaic_gpu_runtime.so"
+if RUNTIME_PATH.exists():
+  # Set this so that the custom call can find it
+  os.environ["MOSAIC_GPU_RUNTIME_LIB_PATH"] = str(RUNTIME_PATH)
 
 
 mosaic_gpu_p = jax.core.Primitive("mosaic_gpu_p")
@@ -106,30 +83,12 @@ def _mosaic_gpu_abstract_eval(*_, module, out_types, gmem_scratch_bytes):
   del module, gmem_scratch_bytes  # Unused.
   return [jax._src.core.ShapedArray(t.shape, t.dtype) for t in out_types]
 
+# TODO(apaszke): Implement a proper system for managing kernel lifetimes
+kernel_idx = itertools.count()
 
 def _mosaic_gpu_lowering_rule(ctx, *args, module, out_types, gmem_scratch_bytes):
   del out_types  # Unused.
-  runtime_path = (
-      pathlib.Path(mosaic_gpu_lib._mosaic_gpu_ext.__file__).parent.parent.parent
-      / "mosaic" / "gpu" / "libmosaic_gpu_runtime.so"
-  )
-  shared_libs = [str(runtime_path)] if runtime_path.exists() else []
-  engine = ExecutionEngine(
-      module, opt_level=3, shared_libs=shared_libs, enable_object_dump=False
-  )
-  ctx.module_context.add_keepalive(engine)
-  launch_func_ptr = ctypes.cast(engine.lookup("main"), ctypes.c_void_p)
-  init_func_ptr = ctypes.cast(engine.lookup("main_init"), ctypes.c_void_p)
-  # Make sure we won't get accidental hits due to address reuse.
-  mosaic_gpu_lib._mosaic_gpu_ext.invalidate_cache(init_func_ptr.value)
-
-  trampoline_args = (ctypes.c_void_p * 2)()
-  trampoline_args[0] = launch_func_ptr
-  trampoline_args[1] = init_func_ptr
-  ctx.module_context.add_keepalive(trampoline_args)
-  ptr_bytes = ctypes.cast(trampoline_args, ctypes.c_void_p).value.to_bytes(
-      8, byteorder="little"
-  )  # pytype: disable=attribute-error
+  idx_bytes = next(kernel_idx).to_bytes(8, byteorder="little")
   op = mlir.custom_call(
       "mosaic_gpu",
       result_types=[
@@ -139,7 +98,8 @@ def _mosaic_gpu_lowering_rule(ctx, *args, module, out_types, gmem_scratch_bytes)
           ),
       ],
       operands=args,
-      backend_config=ptr_bytes,
+      backend_config=idx_bytes
+      + module.operation.get_asm(binary=True, enable_debug_info=True),
   )
   return op.results[:-1]  # Skip the scratch space.
 
@@ -192,7 +152,10 @@ class TileTransform(MemRefTransform):
             arith.divui(i, c(t, index))
             for i, t in zip(idx[-tiling_rank:], self.tiling)
         ),
-        *([c(0, index)] * tiling_rank),
+        *(
+            arith.remui(i, c(t, index))
+            for i, t in zip(idx[-tiling_rank:], self.tiling)
+        ),
     )
 
   def transform_shape(self, shape: Sequence[int]) -> tuple[int, ...]:
@@ -426,7 +389,7 @@ class LaunchContext:
     # nvgpu TMA instructions expect reversed indices...
     rev_dyn_based_indices = reversed(dyn_base_indices)
 
-    uniform_ctx = mgpu.once if uniform else contextlib.nullcontext
+    uniform_ctx = mgpu.single_thread if uniform else contextlib.nullcontext
 
     if gmem_ref is src_ref:
       with uniform_ctx():
@@ -455,6 +418,39 @@ class LaunchContext:
     gpu.barrier()  # Groups are supposedly tracked per-thread
 
 
+# ShapeTrees currently can not contain unions.
+ShapeTree = Any
+RefTree = Any
+T = TypeVar('T')
+
+
+@dataclasses.dataclass(frozen=True)
+class Union(Generic[T]):
+  members: Sequence[T]
+
+
+def _count_buffer_bytes(shape_dtype: jax.ShapeDtypeStruct) -> int:
+  return np.prod(shape_dtype.shape) * np.dtype(shape_dtype.dtype).itemsize
+
+
+def _construct_smem_reftree(
+    dynamic_smem: ir.Value, smem_buffers: ShapeTree) -> RefTree:
+  index = ir.IndexType.get()
+  smem = ir.Attribute.parse("#gpu.address_space<workgroup>")
+  flat_ref_tys, smem_buffer_tree = jax.tree.flatten(smem_buffers)
+  smem_refs = []
+  dynamic_smem_offset = 0
+  for ref_ty in flat_ref_tys:
+    mlir_dtype = mlir.dtype_to_ir_type(ref_ty.dtype)
+    tile_smem = memref.view(
+        ir.MemRefType.get(ref_ty.shape, mlir_dtype, memory_space=smem),
+        dynamic_smem, c(dynamic_smem_offset, index), [],
+    )
+    dynamic_smem_offset += _count_buffer_bytes(ref_ty)
+    smem_refs.append(tile_smem)
+  return jax.tree.unflatten(smem_buffer_tree, smem_refs)
+
+
 # TODO(apaszke): Inline this
 @contextlib.contextmanager
 def _launch(
@@ -462,7 +458,7 @@ def _launch(
     grid,
     block,
     gmem_scratch_ptr,
-    smem_buffers,
+    smem_buffers: ShapeTree | Union[ShapeTree],
     profiler_spec: profiler.ProfilerSpec | None = None,
     maybe_prof_buffer: ir.Value | None = None,
 ):
@@ -473,18 +469,24 @@ def _launch(
   i8 = ir.IntegerType.get_signless(8)
   grid_vals = [c(i, index) for i in grid]
   block_vals = [c(i, index) for i in block]
-  flat_refs, smem_buffer_tree = jax.tree.flatten(smem_buffers)
 
-  smem_ref_bytes = []
-  for ref_ty in flat_refs:
-    smem_ref_bytes.append(
-        np.prod(ref_ty.shape) * np.dtype(ref_ty.dtype).itemsize
-    )
+  if isinstance(smem_buffers, Union):
+    smem_disjoint_live_buffers_collections = smem_buffers.members
+    compute_smem_bytes = max(
+        sum(_count_buffer_bytes(l) for l in jax.tree.leaves(s))
+            for s in smem_buffers.members)
+  else:
+    smem_disjoint_live_buffers_collections = [smem_buffers]
+    compute_smem_bytes = sum(
+        _count_buffer_bytes(l) for l in jax.tree.leaves(smem_buffers))
 
-  smem_bytes = sum(smem_ref_bytes)
+  smem_bytes = compute_smem_bytes
   if profiler_spec is not None:
-    smem_bytes += profiler_spec.smem_bytes(grid)
+    smem_bytes += profiler_spec.smem_bytes(block=block)
 
+  # TODO(cperivol): Query the shared memory size programmatically.
+  if smem_bytes > 228 * 1024:
+    raise ValueError(f"Mosaic GPU kernel exceeds available shared memory {smem_bytes=} > 228000")
   launch_op = gpu.LaunchOp(
       token.type, [token], *grid_vals, *block_vals,
       dynamicSharedMemorySize=c(smem_bytes, i32))
@@ -496,44 +498,45 @@ def _launch(
             (ir.ShapedType.get_dynamic_size(),), i8, memory_space=smem
         )
     )
-    smem_refs = []
-    dynamic_smem_offset = 0
-    for ref_ty, ref_bytes in zip(flat_refs, smem_ref_bytes):
-      mlir_dtype = mlir.dtype_to_ir_type(ref_ty.dtype)
-      tile_smem = memref.view(
-          ir.MemRefType.get(ref_ty.shape, mlir_dtype, memory_space=smem),
-          dynamic_smem, c(dynamic_smem_offset, index), [],
-      )
-      dynamic_smem_offset += ref_bytes
-      smem_refs.append(tile_smem)
+    smem_ref_trees = []
+
+    for smem_live_buffers_collection in smem_disjoint_live_buffers_collections:
+      smem_ref_tree = _construct_smem_reftree(
+          dynamic_smem, smem_live_buffers_collection)
+      smem_ref_trees.append(smem_ref_tree)
 
     if profiler_spec:
       prof_smem = memref.view(
           ir.MemRefType.get(
-              (profiler_spec.smem_i32_elements(grid=grid),),
+              (profiler_spec.smem_i32_elements(block=block),),
               i32, memory_space=smem,
           ),
-          dynamic_smem, c(dynamic_smem_offset, index), [],
+          dynamic_smem, c(compute_smem_bytes, index), [],
       )
       prof = profiler.OnDeviceProfiler(
           profiler_spec, prof_smem, maybe_prof_buffer
       )
     else:
       prof = None
-    smem_ref_tree = jax.tree.unflatten(smem_buffer_tree, smem_refs)
+
+    if isinstance(smem_buffers, Union):
+      smem_ref_tree: Union[RefTree] = Union(smem_ref_trees)
+    else:
+      smem_ref_tree: RefTree = smem_ref_trees[0] if smem_ref_trees else []
+
     yield LaunchContext(launch_op, gmem_scratch_ptr, prof), smem_ref_tree
     if prof is not None:
-      prof.finalize(grid=grid)
+      prof.finalize(grid=grid, block=block)
     gpu.terminator()
 
 
-def as_gpu_kernel(
+def _lower_as_gpu_kernel(
     body,
     grid: tuple[int, ...],
     block: tuple[int, ...],
-    in_shape,
+    in_shapes: tuple[Any, ...],
     out_shape,
-    smem_scratch_shape,
+    smem_scratch_shape: ShapeTree | Union[ShapeTree],
     prof_spec: profiler.ProfilerSpec | None = None,
 ):
   ptr_ty = ir.Type.parse("!llvm.ptr")
@@ -544,11 +547,7 @@ def as_gpu_kernel(
   def _shape_to_ref_ty(shape: jax.ShapeDtypeStruct) -> ir.MemRefType:
     return ir.MemRefType.get(shape.shape, mlir.dtype_to_ir_type(shape.dtype))
 
-  if isinstance(in_shape, list):
-    in_shape = tuple(in_shape)
-  elif not isinstance(in_shape, tuple):
-    in_shape = (in_shape,)
-  in_ref_tys = [_shape_to_ref_ty(t) for t in in_shape]
+  in_ref_tys = [_shape_to_ref_ty(t) for t in in_shapes]
 
   unwrap_output_tuple = False
   if isinstance(out_shape, list):
@@ -558,8 +557,8 @@ def as_gpu_kernel(
     unwrap_output_tuple = True
   out_ref_tys = [_shape_to_ref_ty(t) for t in out_shape]
   if prof_spec is not None:
-    out_shape = (*out_shape, prof_spec.jax_buffer_type)
-    out_ref_tys.append(prof_spec.mlir_buffer_type)
+    out_shape = (*out_shape, prof_spec.jax_buffer_type(grid, block))
+    out_ref_tys.append(prof_spec.mlir_buffer_type(grid, block))
 
   module = ir.Module.create()
   with ir.InsertionPoint(module.body):
@@ -606,22 +605,38 @@ def as_gpu_kernel(
     main.func_op.attributes["llvm.emit_c_interface"] = ir.UnitAttr.get()
   module.operation.verify()
 
+  return module, out_shape, gmem_scratch_bytes, unwrap_output_tuple
+
+
+def as_gpu_kernel(
+    body,
+    grid: tuple[int, ...],
+    block: tuple[int, ...],
+    in_shape,
+    out_shape,
+    smem_scratch_shape: ShapeTree | Union[ShapeTree],
+    prof_spec: profiler.ProfilerSpec | None = None,
+):
+  if isinstance(in_shape, list):
+    in_shape = tuple(in_shape)
+  elif not isinstance(in_shape, tuple):
+    in_shape = (in_shape,)
+
+  module, out_shape, gmem_scratch_bytes, unwrap_output_tuple = (
+      _lower_as_gpu_kernel(
+          body, grid, block, in_shape, out_shape, smem_scratch_shape, prof_spec
+      )
+  )
+
   expected_arg_treedef = jax.tree.structure(in_shape)
-  def _check_args(args):
+  def _check_args(*args):
     arg_treedef = jax.tree.structure(args)
     if arg_treedef != expected_arg_treedef:
       raise ValueError(
           f"Invalid argument structure: expected {expected_arg_treedef}, got"
-          f" {arg_treedef}"
+          f" {arg_treedef}, ({args=})"
       )
 
-  dump_low_level(module)
-
-  pass_manager = _get_mosaic_gpu_pipeline("fatbin")
-  if mosaic_gpu_print_after_all.value:
-    pass_manager.enable_ir_printing()
-  pass_manager.run(module.operation)
-
   def bind(*args):
     return mosaic_gpu_p.bind(
         *args,
@@ -633,7 +648,7 @@ def as_gpu_kernel(
   if prof_spec is not None:
     @jax.jit
     def prof_kernel(*args):
-      _check_args(args)
+      _check_args(*args)
       *results, prof_buffer = bind(*args)
       def dump_profile(prof_buffer):
         out_file = os.path.join(
@@ -642,7 +657,7 @@ def as_gpu_kernel(
         )
         try:
           with open(out_file, "x") as f:
-            prof_spec.dump(prof_buffer, f)
+            prof_spec.dump(prof_buffer, f, grid=grid, block=block)
         except FileExistsError:
           pass  # TODO: Retry
       jax.debug.callback(dump_profile, prof_buffer)
@@ -651,7 +666,7 @@ def as_gpu_kernel(
   else:
     @jax.jit
     def kernel(*args):
-      _check_args(args)
+      _check_args(*args)
       results = bind(*args)
       return results[0] if unwrap_output_tuple else results
     return kernel
@@ -670,82 +685,3 @@ def _declare_runtime_functions():
   func.FuncOp(
       "mosaic_gpu_memcpy_async_h2d", memcpy_async_type, visibility="private"
   )
-
-
-def dump_low_level(module):
-  dump_ptx = mosaic_gpu_dump_ptx.value
-  dump_ptxas = mosaic_gpu_dump_ptxas.value
-  dump_sass = mosaic_gpu_dump_sass.value
-  if not any([dump_ptx, dump_ptxas, dump_sass]):
-    return
-  module = ir.Module.parse(
-      module.operation.get_asm(binary=True, enable_debug_info=True)
-  )
-  pm = _get_mosaic_gpu_pipeline("isa")
-  pm.run(module.operation)
-
-  for op in module.body:
-    if op.OPERATION_NAME == "gpu.binary":
-      objects = ir.ArrayAttr(op.objects)
-      if len(objects) != 1:
-        raise NotImplementedError("Expected a single object")
-      obj = str(objects[0])
-      start = obj.find('assembly = "') + len('assembly = "')
-      end = obj.find('"', start)
-      ptx = obj[start:end]
-      ptx = ptx.replace("\\09", "\t").replace("\\0A", "\n")[:-3]
-      if dump_ptx:
-        print(ptx)
-      if dump_ptxas or dump_sass:
-        with tempfile.TemporaryDirectory() as tmp:
-          ptx_path = os.path.join(tmp, "kernel.ptx")
-          with open(ptx_path, "w") as f:
-            f.write(ptx)
-          elf_path = os.path.join(tmp, 'kernel.o')
-          v_flag = "-v" if dump_ptxas else ""
-          ptxas_flags = f"{v_flag} --opt-level 3 --gpu-name sm_90a"
-          ptxas_out = subprocess.check_output(
-              f"{PTXAS_PATH} {ptxas_flags} --output-file {elf_path} {ptx_path}",
-              stderr=subprocess.STDOUT,
-              shell=True,
-          )
-          if dump_ptxas:
-            print(ptxas_out.decode())
-          if dump_sass:
-            sass = subprocess.check_output(
-                f"{NVDISASM_PATH} -ndf -c {elf_path}",
-                stderr=subprocess.STDOUT,
-                shell=True,
-            )
-            print(sass.decode())
-
-
-def _get_mosaic_gpu_pipeline(kernel_format) -> PassManager:
-  passes = [
-      "convert-nvgpu-to-nvvm",
-      "gpu-kernel-outlining{data-layout-str=}",
-      "convert-vector-to-scf{full-unroll=false lower-tensors=false target-rank=1}",
-      "convert-scf-to-cf",
-      "convert-nvvm-to-llvm",
-      "expand-strided-metadata",
-      "nvvm-attach-target{O=3 chip=sm_90a fast=false features=+ptx80 ftz=false  module= triple=nvptx64-nvidia-cuda}",
-      "lower-affine",
-      "convert-arith-to-llvm{index-bitwidth=0}",
-      "convert-index-to-llvm{index-bitwidth=64}",
-      "canonicalize{max-iterations=10 max-num-rewrites=-1 region-simplify=true test-convergence=false top-down=true}",
-      "cse",
-      "gpu.module(strip-debuginfo)",
-      "gpu.module(convert-gpu-to-nvvm{has-redux=false index-bitwidth=64 use-bare-ptr-memref-call-conv=false})",
-      "gpu.module(canonicalize{max-iterations=10 max-num-rewrites=-1 region-simplify=true test-convergence=false top-down=true})",
-      "gpu.module(cse)",
-      "gpu.module(reconcile-unrealized-casts)",
-      "gpu-to-llvm{gpu-binary-annotation=gpu.binary use-bare-pointers-for-host=false use-bare-pointers-for-kernels=false}",
-      "gpu-module-to-binary{format=" + kernel_format + "}",
-      "convert-math-to-llvm{approximate-log1p=true}",
-      "canonicalize{max-iterations=10 max-num-rewrites=-1 region-simplify=true test-convergence=false top-down=true}",
-      "cse",
-      "reconcile-unrealized-casts",
-      *(["gpu-launch-lowering"] if kernel_format in {"bin", "fatbin"} else []),
-      "convert-func-to-llvm{index-bitwidth=0 use-bare-ptr-memref-call-conv=false}",
-  ]
-  return PassManager.parse(f"builtin.module({','.join(passes)})")
