@@ -37,8 +37,11 @@ from .utils import (
     memref_transpose,
     memref_unfold,
     memref_unsqueeze,
-    once,
+    single_thread,
+    thread_idx,
     tile_shape,
+    warp_idx,
+    warpgroup_idx,
 )
 from .wgmma import (
     WGMMAAccumulator,
