@@ -27,8 +27,8 @@ from jaxlib.mlir.dialects import nvvm
 from jaxlib.mlir.dialects import vector
 import numpy as np
 
-from . import utils
 from . import dsl as mgpu
+from . import utils
 
 # mypy: ignore-errors
 
@@ -36,6 +36,38 @@ WARPGROUP_SIZE = utils.WARPGROUP_SIZE
 c = utils.c
 
 
+@dataclasses.dataclass(frozen=True)
+class WGSplatFragLayout:
+  """A fragmented array where all the values are equal represented as a register per thread.
+
+  FragmentedArrays in this layout can be are always the result of a
+  splat, each thread in the warpgroup has a single copy of the value,
+  while the FragmentedArray pretends it has whatever shape the user
+  wants. This means we can trivially broadcast, reshape and do
+  elementwise operations with all other layouts.
+
+  Example:
+
+  To load a value in
+  ```
+  FragmentedArray.splat(memref.load(ref_1d, [1]), (10,20,2))
+  ```
+
+  A shape is always provided for sanity check reasons.
+
+  """
+
+  shape: tuple[int, ...] = ()
+
+  def can_broadcast_to(self, shape) -> bool:
+    """Check that the shape can be broadcast.
+
+    Only dimensions of size 1 can be broadcast. All other dimensions
+    must be the same as the argument shape.
+    """
+    return all(dim1 == dim2 or dim1 == 1 for dim1, dim2 in zip(self.shape[::-1], shape[::-1]))
+
+
 @dataclasses.dataclass(frozen=True)
 class WGMMAFragLayout:
   """[m, n] matrix, where m % 64 == 0 == n % 8."""
@@ -65,7 +97,15 @@ class WGStridedFragLayout:
     memref_type = ir.MemRefType(memref_ty)
     bw = mgpu.bytewidth(memref_type.element_type)
     assert 8 % bw == 0 and 8 // bw != 0, bw
-    return cls(shape=memref_type.shape, vec_size=8 // bw)
+    if np.prod(memref_type.shape) % WARPGROUP_SIZE != 0:
+      raise ValueError(
+          "Ref must have a number of elements that is a multiple of"
+          f" {WARPGROUP_SIZE}"
+      )
+    max_vec_size = np.prod(memref_type.shape) // WARPGROUP_SIZE
+    return cls(
+        shape=tuple(memref_type.shape), vec_size=min(8 // bw, max_vec_size)
+    )
 
   def thread_vec_idxs(self):
     """The indexes to be used for vector load/store WGStridedFragLayout.
@@ -73,16 +113,17 @@ class WGStridedFragLayout:
     Yields:
       The indices of the vector that correspond to the current thread.
     """
+    index = ir.IndexType.get()
     cardinality = np.prod(self.shape)
     assert cardinality % (WARPGROUP_SIZE * self.vec_size) == 0
     reg_num = cardinality // (WARPGROUP_SIZE * self.vec_size)
-    tidx = gpu.thread_id(gpu.Dimension.x)
+    tidx = arith.remui(gpu.thread_id(gpu.Dimension.x), c(WARPGROUP_SIZE, index))
     off = arith.muli(tidx, c(self.vec_size, tidx.type))
     for i in range(reg_num):
       yield [arith.addi(off, c(i * WARPGROUP_SIZE * self.vec_size, tidx.type))]
 
 
-FragmentedLayout = WGStridedFragLayout | WGMMAFragLayout | WGMMARowFragLayout
+FragmentedLayout = WGSplatFragLayout | WGStridedFragLayout | WGMMAFragLayout | WGMMARowFragLayout
 
 
 WGMMA_LAYOUT = WGMMAFragLayout()
@@ -116,6 +157,12 @@ class FragmentedArray:
         (reg_size,) = ir.VectorType(_registers.flat[0].type).shape
         if np.prod(shape) != np.prod(_registers.shape) * WARPGROUP_SIZE * reg_size:
           raise ValueError((reg_size, shape, _registers.shape, WARPGROUP_SIZE), _registers.flat[0].type)
+
+      # Just a single register
+      case WGSplatFragLayout():
+        if _registers.size != 1:
+          raise ValueError(f"WGStridedFragLayout requires a single value {_registers.shape} ({_registers.size})")
+
       case _:
         raise NotImplementedError
 
@@ -132,7 +179,8 @@ class FragmentedArray:
     return cls(_registers=np.array(vecs), _layout=layout)
 
   @classmethod
-  def splat(cls, value, shape, layout):
+  def splat(cls, value, shape, layout=None):
+    layout = layout or WGSplatFragLayout(shape)
     match layout:
       case WGMMARowFragLayout():
         if len(shape) != 1:
@@ -147,10 +195,14 @@ class FragmentedArray:
           raise ValueError
         reg_shape = (shape[0] // 64, shape[1] // 8, 2, 1)
         value = vector.splat(ir.VectorType.get((2,), value.type), value)
-      case WGStridedFragLayout(shape=shape, vec_size=vec_size):
+      case WGStridedFragLayout(vec_size=vec_size):
+        assert shape == layout.shape
         elems = np.prod(shape)
         reg_shape = (elems // (WARPGROUP_SIZE * vec_size),)
         value = vector.splat(ir.VectorType.get((vec_size,), value.type), value)
+      case WGSplatFragLayout():
+        assert shape == layout.shape
+        reg_shape = ()
       case _:
         raise NotImplementedError(layout)
 
@@ -161,15 +213,17 @@ class FragmentedArray:
 
   @property
   def shape(self):
-    row_tiles = self.registers.shape[0]
     match self.layout:
       case WGMMAFragLayout():
-        col_tiles = self.registers.shape[1]
+        row_tiles, col_tiles = self.registers.shape[:2]
         return (row_tiles * 64, col_tiles * 8)
       case WGMMARowFragLayout():
+        row_tiles = self.registers.shape[0]
         return (row_tiles * 64,)
       case WGStridedFragLayout(shape):
         return shape
+      case WGSplatFragLayout(shape=shape):
+        return shape
 
   @property
   def mlir_dtype(self):
@@ -177,40 +231,73 @@ class FragmentedArray:
     match self.layout:
       case WGMMAFragLayout() | WGStridedFragLayout():
         return ir.VectorType(reg_ty).element_type
-      case WGMMARowFragLayout():
+      case WGMMARowFragLayout() | WGSplatFragLayout():
         return reg_ty
 
   def _pointwise(self, op, *other):
+    other_arrs = []
     for o in other:
       if not isinstance(o, FragmentedArray):
-        return NotImplemented
-      if self.layout != o.layout:
-        raise ValueError("Incompatible FragmentedArray layouts")
-      if self.registers.shape != o.registers.shape:
-        raise ValueError("Incompatible FragmentedArray shapes")
+        if not isinstance(o, ir.Value):
+          raise NotImplementedError(o)
+
+        o = FragmentedArray.splat(o, shape=self.shape, layout=self.layout)
+
+      if isinstance(o.layout, WGSplatFragLayout):
+        if not o.layout.can_broadcast_to(self.shape):
+          raise ValueError("Can't broadcast shape.")
+        o = FragmentedArray.splat(o.registers.flat[0], shape=self.shape, layout=self.layout)
+      else:
+        if self.layout != o.layout:
+          raise ValueError("Incompatible FragmentedArray layouts")
+        if self.registers.shape != o.registers.shape:
+          raise ValueError("Incompatible FragmentedArray shapes")
+
+      other_arrs.append(o)
     new_regs = np.empty_like(self.registers)
+
     for idx, reg in np.ndenumerate(self.registers):
-      new_regs[idx] = op(reg, *(o.registers[idx] for o in other))
+      new_regs[idx] = op(reg, *(o.registers[idx] for o in other_arrs))
     return FragmentedArray(_registers=new_regs, _layout=self.layout)
 
   def __add__(self, other):
-    return self._pointwise(arith.addf, other)
+    if ir.FloatType.isinstance(self.mlir_dtype):
+      return self._pointwise(arith.addf, other)
+    elif ir.IntegerType.isinstance(self.mlir_dtype):
+      return self._pointwise(arith.addi, other)
+    else:
+      raise NotImplementedError(self.mlir_dtype)
 
   def __mul__(self, other):
-    return self._pointwise(arith.mulf, other)
+    if ir.FloatType.isinstance(self.mlir_dtype):
+      return self._pointwise(arith.mulf, other)
+    elif ir.IntegerType.isinstance(self.mlir_dtype):
+      return self._pointwise(arith.muli, other)
+    else:
+      raise NotImplementedError(self.mlir_dtype)
 
   def __sub__(self, other):
+    if not ir.FloatType.isinstance(self.mlir_dtype):
+      raise NotImplementedError
     return self._pointwise(arith.subf, other)
 
   def __truediv__(self, other):
+    if not ir.FloatType.isinstance(self.mlir_dtype):
+      raise NotImplementedError
     return self._pointwise(arith.divf, other)
 
   def max(self, other):
+    if not ir.FloatType.isinstance(self.mlir_dtype):
+      raise NotImplementedError
     return self._pointwise(arith.maximumf, other)
 
   def exp(self, approx: bool = False):
+    if not ir.FloatType.isinstance(self.mlir_dtype):
+      raise NotImplementedError
     def fast_exp(x):
       f32 = ir.F32Type.get()
+      if self.mlir_dtype != f32:
+        raise NotImplementedError
       log2e = arith.constant(f32, ir.FloatAttr.get(f32, 1.4426950408889634))
       if x.type == f32:
         scaled = arith.mulf(x, log2e)
@@ -229,6 +316,9 @@ class FragmentedArray:
         raise NotImplementedError(x.type)
     return self._pointwise(fast_exp if approx else mlir_math.exp)
 
+  def rsqrt(self):
+    return self._pointwise(mlir_math.rsqrt)
+
   def __and__(self, other):
     if not ir.IntegerType.isinstance(self.mlir_dtype):
       raise ValueError(
@@ -300,7 +390,7 @@ class FragmentedArray:
         new_reg_ty = ir.VectorType.get((2,), new_dtype)
       case WGStridedFragLayout(vec_size=vec_size):
         new_reg_ty = ir.VectorType.get((vec_size,), new_dtype)
-      case WGMMARowFragLayout():
+      case WGMMARowFragLayout() | WGSplatFragLayout():
         new_reg_ty = new_dtype
       case _:
         raise NotImplementedError(f"Unsupported layout {self.layout}")
@@ -308,6 +398,45 @@ class FragmentedArray:
       new_registers[idx] = convert(new_reg_ty, reg)
     return FragmentedArray(_registers=new_registers, _layout=self.layout)
 
+  def reduce_sum(self, scratch) -> ir.Value:
+    index = ir.IndexType.get()
+    if not isinstance(self.layout, WGStridedFragLayout):
+      raise NotImplementedError(f"Unsupported layout {self.layout}")
+    result = c(0, self.mlir_dtype)
+    for reg in self.registers:
+      result = arith.addf(
+          result,
+          vector.reduction(self.mlir_dtype, vector.CombiningKind.ADD, reg),
+      )
+    scratch_ty = ir.MemRefType(scratch.type)
+    if scratch_ty.element_type != self.mlir_dtype or scratch_ty.shape != [4]:
+      raise ValueError(f"Expected shape={(4,)}, {self.mlir_dtype} (got {scratch_ty})")
+
+    if ir.FloatType.isinstance(self.mlir_dtype):
+      op = arith.addf
+    elif ir.IntegerType.isinstance(self.mlir_dtype):
+      op = arith.addi
+    else:
+      raise NotImplementedError(self.mlir_dtype)
+
+    warp_result = utils.warp_tree_reduce(result, op, 32)
+    warp_id = arith.divui(gpu.thread_id(gpu.Dimension.x), c(32, index))
+    memref.store(warp_result, scratch, [warp_id])
+    utils.commit_shared()
+    zero_index = c(0, index)
+    with mgpu.single_thread():
+      scratch_vec = vector.load(
+          ir.VectorType.get((4,), self.mlir_dtype),
+          scratch,
+          [zero_index],
+      )
+      scratch_sum = vector.reduction(
+          self.mlir_dtype, vector.CombiningKind.ADD, scratch_vec
+      )
+      memref.store(scratch_sum, scratch, [zero_index])
+    utils.commit_shared()
+    return memref.load(scratch, [zero_index])
+
   def reduce(self, op, axis):
     if self.layout != WGMMA_LAYOUT:
       raise NotImplementedError(self.layout)
@@ -343,6 +472,30 @@ class FragmentedArray:
       new_regs[row_tile, row_subtile] = result
     return FragmentedArray(_registers=new_regs, _layout=WGMMA_ROW_LAYOUT)
 
+  def broadcast(self, shape):
+    if not isinstance(self.layout, WGSplatFragLayout):
+      raise NotImplementedError(self.layout)
+
+    if self.shape == shape:
+      return self
+
+    if not self.layout.can_broadcast_to(shape):
+      raise ValueError(f"Can't broadcast {self.shape} to {shape}")
+
+    return FragmentedArray(_registers=self.registers, _layout=WGSplatFragLayout(shape))
+
+  def reshape(self, shape):
+    if self.shape == shape:
+      return self
+
+    if not isinstance(self.layout, WGSplatFragLayout):
+      raise NotImplementedError(self.layout)
+
+    if np.prod(shape) != np.prod(self.shape):
+      raise ValueError(f"Can't reshape {self.shape} to {shape}")
+
+    return FragmentedArray(_registers=self.registers, _layout=WGSplatFragLayout(shape))
+
   def broadcast_minor(self, n):
     if self.layout != WGMMA_ROW_LAYOUT:
       raise NotImplementedError
@@ -372,8 +525,9 @@ class FragmentedArray:
 
   def _store_untiled_wg_strided(self, ref: ir.Value):
     ref_ty = ir.MemRefType(ref.type)
-    if ref_ty.shape != self.shape:
-      raise ValueError((ref_ty.shape, self.shape))
+    ref_shape = tuple(ref_ty.shape)
+    if ref_shape != self.shape:
+      raise ValueError((ref_shape, self.shape))
     smem_1d = mgpu.memref_fold(ref, 0, len(ref_ty.shape))
     for idx, reg in zip(self.layout.thread_vec_idxs(), self.registers.flat):
       vector.store(reg, smem_1d, idx)
@@ -390,7 +544,7 @@ class FragmentedArray:
     def c(x):
       return arith.ConstantOp(index, ir.IntegerAttr.get(index, x))
 
-    tidx = gpu.thread_id(gpu.Dimension.x)
+    tidx = arith.remui(gpu.thread_id(gpu.Dimension.x), c(WARPGROUP_SIZE))
     lane_id = arith.remui(tidx, c(32))  # {0, 1, ..., 31}
     warp_id = arith.divui(tidx, c(32))  # {0, 1, 2, 3}
     row_base = arith.addi(
@@ -442,19 +596,15 @@ class FragmentedArray:
   @staticmethod
   def transfer_tiled(shape, dtype, swizzle: int | None):
     bw = mgpu.bytewidth(dtype)
-    cols_per_tile = 128 // bw
     m, n = shape
     if n % 32 != 0:
       raise NotImplementedError
     cols_per_tile = 128 // bw
     if swizzle != 128:
       raise NotImplementedError("Only 128B swizzle supported")
-    index = ir.IndexType.get()
 
-    def c(x):
-      return arith.ConstantOp(index, ir.IntegerAttr.get(index, x))
-
-    tidx = gpu.thread_id(gpu.Dimension.x)
+    c = arith.ConstantOp.create_index
+    tidx = arith.remui(gpu.thread_id(gpu.Dimension.x), c(WARPGROUP_SIZE))
     lane_id = arith.remui(tidx, c(32))  # {0, 1, ..., 31}
     warp_id = arith.divui(tidx, c(32))  # {0, 1, 2, 3}
     sub_row_base = arith.divui(lane_id, c(4))  # {0, 1, ..., 7}
@@ -465,7 +615,7 @@ class FragmentedArray:
     else:
       # We rely on canonicalization to clean up the selects.
       i1 = ir.IntegerType.get_signless(1)
-      is_even_row = arith.constant(i1, ir.IntegerAttr.get(i1, 1))
+      is_even_row = arith.constant(i1, ir.BoolAttr.get(True))
     row_base = arith.addi(sub_row_base, arith.muli(warp_id, c(16)))
     col_base = arith.muli(arith.remui(lane_id, c(4)), c(2))  # {0, 2, 4, 6}
     # The swizzle pattern is constant for a given thread.
