@@ -14,11 +14,11 @@
 # ==============================================================================
 """Utilities for code generator."""
 
+from collections.abc import Iterator
 import contextlib
 import dataclasses
 from typing import Any, Literal, Sequence
 
-from absl import flags
 import jax
 from jaxlib.mlir import ir
 from jaxlib.mlir.dialects import arith
@@ -37,10 +37,6 @@ import numpy as np
 WARPGROUP_SIZE: int = 128
 DYNAMIC = -9223372036854775808
 
-FLAGS = flags.FLAGS
-
-flags.DEFINE_bool("mosaic_gpu_debug", False, "Perform debug printing")
-
 # pylint: disable=line-too-long, wildcard-import, missing-function-docstring, bad-continuation, g-bad-todo, protected-access, g-explicit-length-test, missing-class-docstring, g-doc-return-or-yield, g-inconsistent-quotes
 
 
@@ -112,8 +108,6 @@ def get_tensormap_descriptor(**attrs):
 
 
 def debug_print(fmt, *args, uniform=True):
-  if not FLAGS.mosaic_gpu_debug:
-    return
   type_formats = []
   new_args = []
   for arg in args:
@@ -136,7 +130,7 @@ def debug_print(fmt, *args, uniform=True):
       raise NotImplementedError(arg.type)
     type_formats.append(ty_format)
     new_args.append(arg)
-  ctx = once if uniform else contextlib.nullcontext
+  ctx = single_thread if uniform else contextlib.nullcontext
   with ctx():
     gpu.printf(fmt.format(*type_formats) + "\n", new_args)
 
@@ -185,33 +179,55 @@ def fori(bound, carrys):
   return wrapper
 
 
-def get_warp_idx():
+def thread_idx():
+  i32 = ir.IntegerType.get_signless(32)
+  as_i32 = lambda x: arith.index_cast(i32, x)
+  tidx = as_i32(gpu.thread_id(gpu.Dimension.x))
+  stride = as_i32(gpu.block_dim(gpu.Dimension.x))
+  for dim in (gpu.Dimension.y, gpu.Dimension.z):
+    tidx = arith.addi(tidx, arith.muli(as_i32(gpu.thread_id(dim)), stride))
+    stride = arith.muli(stride, as_i32(gpu.block_dim(dim)))
+  return tidx
+
+
+def _warp_bcast(val, lane_idx=0):
   i32 = ir.IntegerType.get_signless(32)
-  tidx = arith.index_cast(i32, gpu.thread_id(gpu.Dimension.x))
-  warp_idx = arith.shrui(tidx, c(5, tidx.type))
   mask = c(0xFFFFFFFF, i32)
   return nvvm.shfl_sync(
-      warp_idx.type, mask, warp_idx, c(0, i32), c(0x1F, i32), nvvm.ShflKind.idx
+      val.type, mask, val, c(lane_idx, i32), c(0x1F, i32), nvvm.ShflKind.idx
   )
 
 
+def warp_idx(sync=True):
+  i32 = ir.IntegerType.get_signless(32)
+  warp_idx = arith.shrui(thread_idx(), c(5, i32))
+  # Performing a warp broadcast improves performance as compiler understands
+  # that the value is uniform across the warp.
+  return _warp_bcast(warp_idx) if sync else warp_idx
+
+
+def warpgroup_idx(sync=True):
+  i32 = ir.IntegerType.get_signless(32)
+  wg_idx = arith.shrui(thread_idx(), c(7, i32))
+  # Performing a warp broadcast improves performance as compiler understands
+  # that the value is uniform across the warp.
+  return _warp_bcast(wg_idx) if sync else wg_idx
+
+
 # True withon `once()` contexts.
 _ONCE_REGION_ACTIVE = False
 
 
 @contextlib.contextmanager
-def once():
-  """Runs the context only from a single thread from the first warp.
-
-  The block is assumed to have a size of 1 in both y and z dimensions.
-  """
+def single_thread():
+  """Runs the context only from a single thread."""
   global _ONCE_REGION_ACTIVE
 
   if _ONCE_REGION_ACTIVE:
     yield
     return
 
-  warp = get_warp_idx()
+  warp = warp_idx()
   first_warp = arith.cmpi(arith.CmpIPredicate.eq, warp, c(0, warp.type))
   elected = nvvm.elect_sync(ir.IntegerType.get_signless(1))
   should_run = arith.andi(first_warp, elected)
@@ -314,7 +330,9 @@ def memref_fold(ref: ir.Value, dim, fold_rank) -> ir.Value:
   new_shape = list(ref_ty.shape)
   new_shape[dim : dim + fold_rank] = [np.prod(new_shape[dim : dim + fold_rank])]
   identity = ir.AffineMapAttr.get(ir.AffineMap.get_identity(ref_ty.rank))
-  if ref_ty.layout == identity:
+  contig_strided_1d = ir.Attribute.parse("strided<[1]>")
+  # Not sure why but MLIR expects the strided 1D layout to disappear in this op.
+  if ref_ty.layout == identity or ref_ty.layout == contig_strided_1d:
     new_layout = ir.AffineMapAttr.get(
         ir.AffineMap.get_identity(ref_ty.rank - fold_rank + 1)
     )
@@ -399,7 +417,7 @@ def memref_unsqueeze(ref: ir.Value, dim) -> ir.Value:
     )
     assoc = [[d] for d in range(ref_ty.rank)]
     assoc[-1].append(ref_ty.rank)
-    return memref.expand_shape(new_ty, ref, assoc)
+    return memref.expand_shape(new_ty, ref, assoc, [], new_ty.shape)
   else:
     return memref_unfold(ref, dim, (1, None))
 
@@ -466,22 +484,29 @@ def commit_shared():
 
 class BarrierArray:
 
-  def __init__(self, num_barriers):
+  def __init__(self, num_barriers: int, arrival_count: int = 1):
     barrier_group_ty = ir.Type.parse(
         "!nvgpu.mbarrier.group<memorySpace=#gpu.address_space<workgroup>,"
         f" num_barriers={num_barriers}>"
     )
 
+    self.num_barriers = num_barriers
     self.value = nvgpu.mbarrier_create(barrier_group_ty)
+    self.num_barriers = num_barriers
     index = ir.IndexType.get()
     if num_barriers > 32:
       raise NotImplementedError("Only up to 32 barriers per group supported")
     i32 = ir.IntegerType.get_signless(32)
     self.phases = memref.alloca(ir.MemRefType.get((), i32), [], [])
     memref.store(c(0, i32), self.phases, [])
-    with once():
+    with single_thread():
       for i in range(num_barriers):
-        nvgpu.mbarrier_init(self.value, c(1, index), c(i, index))
+        nvgpu.mbarrier_init(self.value, c(arrival_count, index), c(i, index))
+    gpu.barrier()
+
+  def __iter__(self) -> Iterator["Barrier"]:
+    for offset in range(self.num_barriers):
+      yield self[offset]
 
   def __getitem__(self, offset: ir.Value | int):
     if isinstance(offset, int):
@@ -512,6 +537,10 @@ class Barrier:
     memref.store(new_parities, self.barrier_array.phases, [])
     self.wait_parity(parity)
 
+  def arrive(self):
+    token_ty = ir.Type.parse("!nvgpu.mbarrier.token")
+    nvgpu.mbarrier_arrive(token_ty, self.barrier_array.value, self.offset)
+
 
 class Partition:
   source_bounds: tuple[int, ...]
@@ -645,3 +674,26 @@ def tile_shape(shape, tiling):
       *(s // t for s, t in zip(shape[-tiling_rank:], tiling)),
       *tiling,
   )
+
+
+def warp_tree_reduce(value, op, group_size):
+  """Reduce a value across the warpgroup."""
+  assert 32 % group_size == 0 and group_size <= 32
+  i32 = ir.IntegerType.get_signless(32)
+  result = value
+  iters = np.log2(group_size)
+  if not iters.is_integer():
+    raise ValueError(f"Warp reduction group size should be a power of 2 (got {group_size})")
+  iters = int(iters)
+  for i in range(iters):
+    other_result = nvvm.shfl_sync(
+        result.type,
+        c(0xFFFFFFFF, i32),
+        result,
+        c(1 << i, i32),
+        c(0x1F, i32),
+        nvvm.ShflKind.bfly,
+    )
+    result = op(result, other_result)
+
+  return result
